import threading
import time as mod_time
import uuid
from .exceptions import LockError, LockNotOwnedError
from .utils import dummy


class Lock(object):
    """
    A shared, distributed Lock. Using Redis for locking allows the Lock
    to be shared across processes and/or machines.

    It's left to the user to resolve deadlock issues and make sure
    multiple clients play nicely together.
    """

    lua_release = None
    lua_extend = None
    lua_reacquire = None

    # KEYS[1] - lock name
    # ARGS[1] - token
    # return 1 if the lock was released, otherwise 0
    LUA_RELEASE_SCRIPT = """
        local token = redis.call('get', KEYS[1])
        if not token or token ~= ARGV[1] then
            return 0
        end
        redis.call('del', KEYS[1])
        return 1
    """

    # KEYS[1] - lock name
    # ARGS[1] - token
    # ARGS[2] - additional milliseconds
    # return 1 if the locks time was extended, otherwise 0
    LUA_EXTEND_SCRIPT = """
        local token = redis.call('get', KEYS[1])
        if not token or token ~= ARGV[1] then
            return 0
        end
        local expiration = redis.call('pttl', KEYS[1])
        if not expiration then
            expiration = 0
        end
        if expiration < 0 then
            return 0
        end
        redis.call('pexpire', KEYS[1], expiration + ARGV[2])
        return 1
    """

    # KEYS[1] - lock name
    # ARGS[1] - token
    # ARGS[2] - milliseconds
    # return 1 if the locks time was reacquired, otherwise 0
    LUA_REACQUIRE_SCRIPT = """
        local token = redis.call('get', KEYS[1])
        if not token or token ~= ARGV[1] then
            return 0
        end
        redis.call('pexpire', KEYS[1], ARGV[2])
        return 1
    """

    def __init__(self, redis, name, timeout=None, sleep=0.1,
                 blocking=True, blocking_timeout=None, thread_local=True):
        """
        Create a new Lock instance named ``name`` using the Redis client
        supplied by ``redis``.

        ``timeout`` indicates a maximum life for the lock.
        By default, it will remain locked until release() is called.
        ``timeout`` can be specified as a float or integer, both representing
        the number of seconds to wait.

        ``sleep`` indicates the amount of time to sleep per loop iteration
        when the lock is in blocking mode and another client is currently
        holding the lock.

        ``blocking`` indicates whether calling ``acquire`` should block until
        the lock has been acquired or to fail immediately, causing ``acquire``
        to return False and the lock not being acquired. Defaults to True.
        Note this value can be overridden by passing a ``blocking``
        argument to ``acquire``.

        ``blocking_timeout`` indicates the maximum amount of time in seconds to
        spend trying to acquire the lock. A value of ``None`` indicates
        continue trying forever. ``blocking_timeout`` can be specified as a
        float or integer, both representing the number of seconds to wait.

        ``thread_local`` indicates whether the lock token is placed in
        thread-local storage. By default, the token is placed in thread local
        storage so that a thread only sees its token, not a token set by
        another thread. Consider the following timeline:

            time: 0, thread-1 acquires `my-lock`, with a timeout of 5 seconds.
                     thread-1 sets the token to "abc"
            time: 1, thread-2 blocks trying to acquire `my-lock` using the
                     Lock instance.
            time: 5, thread-1 has not yet completed. redis expires the lock
                     key.
            time: 5, thread-2 acquired `my-lock` now that it's available.
                     thread-2 sets the token to "xyz"
            time: 6, thread-1 finishes its work and calls release(). if the
                     token is *not* stored in thread local storage, then
                     thread-1 would see the token value as "xyz" and would be
                     able to successfully release the thread-2's lock.

        In some use cases it's necessary to disable thread local storage. For
        example, if you have code where one thread acquires a lock and passes
        that lock instance to a worker thread to release later. If thread
        local storage isn't disabled in this case, the worker thread won't see
        the token set by the thread that acquired the lock. Our assumption
        is that these cases aren't common and as such default to using
        thread local storage.
        """
        self.redis = redis
        self.name = name
        self.timeout = timeout
        self.sleep = sleep
        self.blocking = blocking
        self.blocking_timeout = blocking_timeout
        self.thread_local = bool(thread_local)
        self.local = threading.local() if self.thread_local else dummy()
        self.local.token = None
        if self.timeout and self.sleep > self.timeout:
            raise LockError("'sleep' must be less than 'timeout'")
        self.register_scripts()

    def register_scripts(self):
        cls = self.__class__
        client = self.redis
        if cls.lua_release is None:
            cls.lua_release = client.register_script(cls.LUA_RELEASE_SCRIPT)
        if cls.lua_extend is None:
            cls.lua_extend = client.register_script(cls.LUA_EXTEND_SCRIPT)
        if cls.lua_reacquire is None:
            cls.lua_reacquire = \
                client.register_script(cls.LUA_REACQUIRE_SCRIPT)

    def __enter__(self):
        # force blocking, as otherwise the user would have to check whether
        # the lock was actually acquired or not.
        if self.acquire(blocking=True):
            return self
        raise LockError("Unable to acquire lock within the time specified")

    def __exit__(self, exc_type, exc_value, traceback):
        self.release()

    def acquire(self, blocking=None, blocking_timeout=None, token=None):
        """
        Use Redis to hold a shared, distributed lock named ``name``.
        Returns True once the lock is acquired.

        If ``blocking`` is False, always return immediately. If the lock
        was acquired, return True, otherwise return False.

        ``blocking_timeout`` specifies the maximum number of seconds to
        wait trying to acquire the lock.

        ``token`` specifies the token value to be used. If provided, token
        must be a bytes object or a string that can be encoded to a bytes
        object with the default encoding. If a token isn't specified, a UUID
        will be generated.
        """
        sleep = self.sleep
        if token is None:
            token = uuid.uuid1().hex.encode()
        else:
            encoder = self.redis.connection_pool.get_encoder()
            token = encoder.encode(token)
        if blocking is None:
            blocking = self.blocking
        if blocking_timeout is None:
            blocking_timeout = self.blocking_timeout
        stop_trying_at = None
        if blocking_timeout is not None:
            stop_trying_at = mod_time.time() + blocking_timeout
        while True:
            if self.do_acquire(token):
                self.local.token = token
                return True
            if not blocking:
                return False
            if stop_trying_at is not None and mod_time.time() > stop_trying_at:
                return False
            mod_time.sleep(sleep)

    def do_acquire(self, token):
        if self.timeout:
            # convert to milliseconds
            timeout = int(self.timeout * 1000)
        else:
            timeout = None
        if self.redis.set(self.name, token, nx=True, px=timeout):
            return True
        return False

    def locked(self):
        """
        Returns True if this key is locked by any process, otherwise False.
        """
        return self.redis.get(self.name) is not None

    def owned(self):
        """
        Returns True if this key is locked by this lock, otherwise False.
        """
        stored_token = self.redis.get(self.name)
        # need to always compare bytes to bytes
        # TODO: this can be simplified when the context manager is finished
        if stored_token and not isinstance(stored_token, bytes):
            encoder = self.redis.connection_pool.get_encoder()
            stored_token = encoder.encode(stored_token)
        return self.local.token is not None and \
            stored_token == self.local.token

    def release(self):
        "Releases the already acquired lock"
        expected_token = self.local.token
        if expected_token is None:
            raise LockError("Cannot release an unlocked lock")
        self.local.token = None
        self.do_release(expected_token)

    def do_release(self, expected_token):
        if not bool(self.lua_release(keys=[self.name],
                                     args=[expected_token],
                                     client=self.redis)):
            raise LockNotOwnedError("Cannot release a lock"
                                    " that's no longer owned")

    def extend(self, additional_time):
        """
        Adds more time to an already acquired lock.

        ``additional_time`` can be specified as an integer or a float, both
        representing the number of seconds to add.
        """
        if self.local.token is None:
            raise LockError("Cannot extend an unlocked lock")
        if self.timeout is None:
            raise LockError("Cannot extend a lock with no timeout")
        return self.do_extend(additional_time)

    def do_extend(self, additional_time):
        additional_time = int(additional_time * 1000)
        if not bool(self.lua_extend(keys=[self.name],
                                    args=[self.local.token, additional_time],
                                    client=self.redis)):
            raise LockNotOwnedError("Cannot extend a lock that's"
                                    " no longer owned")
        return True

    def reacquire(self):
        """
        Resets a TTL of an already acquired lock back to a timeout value.
        """
        if self.local.token is None:
            raise LockError("Cannot reacquire an unlocked lock")
        if self.timeout is None:
            raise LockError("Cannot reacquire a lock with no timeout")
        return self.do_reacquire()

    def do_reacquire(self):
        timeout = int(self.timeout * 1000)
        if not bool(self.lua_reacquire(keys=[self.name],
                                       args=[self.local.token, timeout],
                                       client=self.redis)):
            raise LockNotOwnedError("Cannot reacquire a lock that's"
                                    " no longer owned")
        return True



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .filetype import *  # noqa
from .helpers import *  # noqa
from .match import *  # noqa

# Current package semver version
__version__ = version = '1.0.5'



# -*- coding: utf-8 -*-
import time,hashlib,json,re,os,platform,sys,shutil,requests,importlib,traceback,pip,gzip,tarfile,zipfile,random,copy
import datetime as core_datetime
from kcweb import config
from kcweb.utill.dateutil.relativedelta import relativedelta as core_relativedelta
from kcweb.utill.db import mysql as kcwmysql
from kcweb.utill.db import mongodb as kcwmongodb
from kcweb.utill.db import sqlite as kcwsqlite
from kcweb.utill.cache import cache as kcwcache
from kcweb.utill.redis import redis as kcwredis
from kcweb.utill.http import Http
from kcweb.utill.queues import Queues
from kcweb.utill.db import model
from mako.template import Template as kcwTemplate
from mako.lookup import TemplateLookup
import smtplib
from email.mime.text import MIMEText
from email.utils import formataddr
from kcweb.utill import filetype
from . import globals

import asyncio,websockets,urllib,threading
class kcwebsocket:
    "websocket服务端"
    __clientlists=[] #所有客户端绑定的对象
    __lists=[] #所有客户端
    __group={} #组
    __uid={} #clientid绑定的uid
    async def bindUid(self,clientid,uid):
        """将clientid与uid绑定，以便通过sendToUid(uid)发送数据

        clientid 客户端id

        uid uid与client_id是一对多的关系，系统允许一个uid下有多个client_id
        """
        try:
            self.__uid[uid]
        except KeyError:
            self.__uid[uid]=[]
        self.__uid[uid].append(clientid)
    def unbindUid(self,clientid,uid):
        """将clientid与uid解绑 当clientid下线（连接断开）时会自动与uid解绑，开发者无需在onClose事件调用unbindUid
        
        clientid 客户端id

        uid 数字或者字符串
        """
        try:
            self.__uid[uid]
        except KeyError:
            pass
        else:
            try:
                self.__uid[uid].remove(clientid)
            except KeyError:
                pass
    async def sendToUid(self,uid,message):
        """向uid绑定的所有在线clientid发送数据

        uid uid可以是字符串、数字、或者包含uid的列表。如果为列表，则是给列表内所有uid发送数据

        message 要发送的数据（字符串类型）
        """
        if isinstance(uid,str):
            for clientid in self.__uid[uid]:
                await self.send_client(clientid,message)
        elif isinstance(uid,list):
            for k in uid:
                for clientid in self.__uid[k ]:
                    try:
                        await self.send_client(clientid,message)
                    except ValueError:
                        if config.app['app_debug']:
                            print("deluid",clientid)
                        self.__uid[uid].remove(clientid)
    async def joinGroup(self,clientid,group):
        """将clientid加入某个组
        
        clientid 客户端id

        Group 组名
        """
        try:
            self.__group[group]
        except KeyError:
            self.__group[group]=[]
        self.__group[group].append(clientid)
    async def leaveGroup(self,clientid,group):
        """将clientid从某个组中删除
        
        clientid 客户端id

        group 组名
        """
        try:
            self.__group[group]
        except KeyError:
            pass
        else:
            try:
                self.__group[group].remove(clientid)
            except KeyError:
                pass
    async def ungroup(self,group):
        """解散分组。 解散分组后所有属于这个分组的用户的连接将被移出分组，此分组将不再存在，除非再次调用 joinGroup

        group 组名
        """
        try:
            self.__group[group]
        except KeyError:
            pass
        else:
            del self.__group[group]
    async def sendToGroup(self,group,message,exclude_clientid=[]):
        """向某个分组的所有在线clientid发送数据。

        group 组名

        message 要发送的数据（字符串类型）

        exclude_clientid clientid组成的列表。exclude_clientid列表中指定的clientid将被排除在外，不会收到本次发的消息
        """
        try:
            self.__group[group]
        except KeyError:
            pass
        else:
            for client in self.__group[group]:
                if exclude_clientid:
                    for clientid in exclude_clientid:
                        if client!=clientid:
                            try:
                                await self.send_client(client,message)
                            except ValueError:
                                if config.app['app_debug']:
                                    print("delgroup",client)
                                self.__group[group].remove(client)
                else:
                    try:
                        await self.send_client(client,message)
                    except ValueError:
                        if config.app['app_debug']:
                            print("delgroup",client)
                        self.__group[group].remove(client)
    async def getClientIdCountByGroup(self,group):
        """获取某分组当前在线成连接数（多少clientid在线）
        
        group 组名

        return int 返回一个数字
        """
        try:
            self.__group[group]
        except KeyError:
            return 0
        else:
            return len(self.__group[group])
    def getAllClientIdCount(self):
        """获取当前在线连接总数（多少client_id在线）
        
        return int 返回一个数字
        """
        return len(self.__lists)
    async def send_all(self,message):
        "给所有人发送消息，包括自己"
        for l in self.__clientlists:
            try:
                await l['socket'].send(message)
            except:pass
    async def send_client(self,clientid,message):
        "给所指定客户端发送消息"
        xb=self.__lists.index(clientid)
        websockets=self.__clientlists[xb]['socket']
        try:
            await websockets.send(message)
        except:pass
    async def onConnect(self,clientid,params):
        "客户端发来连接时"
        if config.app['app_debug']:
            print("连接成功",clientid)
    async def onMessage(self,clientid,recv_text):
        "当客户端发来数据"
        await self.send_client(clientid,recv_text) #给当前用户发送消息
    async def onClose(self,clientid):
        "客户端与websocket的连接断开时触发"
        if config.app['app_debug']:
            print("onClose",clientid)
    async def __onClose(self,clientid):
        xb=self.__lists.index(clientid)
        del self.__lists[xb]
        del self.__clientlists[xb]
        if self.__uid:
            for uid in self.__uid.keys():
                try:
                    self.__uid[uid]
                except KeyError:
                    pass
                else:
                    try:
                        self.__uid[uid]['clientid']
                    except KeyError:
                        pass
                    else:
                        self.__uid[uid].remove(clientid)
    async def __main2(self,clientid,websocket,path):
        "服务器端主逻辑"
        async for message in websocket:
            await self.onMessage(clientid, message)
        await self.__onClose(clientid)
        await self.onClose(clientid)
    async def __main1(self,clientid,websocket,path):
        t = urllib.parse.parse_qs(urllib.parse.urlparse(path).query)
        params={}
        for key in t.keys():
            params[key]=t[key][0]
        await self.onConnect(clientid, params)
    async def __main(self,websocket,path):
        "服务器端主逻辑"
        clientid=md5(str(random.random()))
        self.__clientlists.append({"clientid":clientid,"socket":websocket})
        self.__lists.append(clientid)
        task1=asyncio.ensure_future(self.__main1(clientid,websocket,path))
        task2=asyncio.ensure_future(self.__main2(clientid,websocket,path))
        await task1
        await task2
    def start(self,ip='0.0.0.0',port='39020'):
        "启动websoeket服务"
        asyncio.set_event_loop(asyncio.new_event_loop()) # 防止出现RuntimeError
        asyncio.get_event_loop().run_until_complete(websockets.serve(self.__main,ip,port))
        asyncio.get_event_loop().run_forever()
# def start():
#     kwebsocket=kcwebsocket()
#     kwebsocket.start()


class DFAFilter():
    """DFA算法文字过滤器"""
    __sensitivelist=[]
    def __init__(self,sensitivelist):
        """
        sensitivelist 敏感字符串列表
        """
        self.keyword_chains = {}  # 关键词链表
        self.delimit = '\x00'  # 限定
        for keyword in sensitivelist:
            self.__add(str(keyword).strip())
    def __add(self, keyword):
        keyword = keyword.lower()  # 关键词英文变为小写
        chars = keyword.strip()  # 关键字去除首尾空格和换行
        if not chars:  # 如果关键词为空直接返回
            return
        level = self.keyword_chains
        # 遍历关键字的每个字
        for i in range(len(chars)):
            # 如果这个字已经存在字符链的key中就进入其子字典
            if chars[i] in level:
                level = level[chars[i]]
            else:
                if not isinstance(level, dict):
                    break
                for j in range(i, len(chars)):
                    level[chars[j]] = {}
                    last_level, last_char = level, chars[j]
                    level = level[chars[j]]
                last_level[last_char] = {self.delimit: 0}
                break
        if i == len(chars) - 1:
            level[self.delimit] = 0
    def filter(self, message, repl=""):
        """ 获取过滤后的字符串

        message 待处理的字符串

        repl 替换内容
        """
        message = message.lower()
        ret = []
        start = 0
        while start < len(message):
            level = self.keyword_chains
            step_ins = 0
            for char in message[start:]:
                if char in level:
                    step_ins += 1
                    if self.delimit not in level[char]:
                        level = level[char]
                    else:
                        ret.append(repl * step_ins)
                        start += step_ins - 1
                        break
                else:
                    ret.append(message[start])
                    break
            else:
                ret.append(message[start])
            start += 1
        return ''.join(ret)
    def sensitive(self,message):
        """ 判断字符串是否包含敏感

        message 待处理的字符串
        """
        if self.filter(message) == message:
            return True
        else:
            return False
redis=kcwredis()
def send_mail(user,text="邮件内容",theme="邮件主题",recNick="收件人昵称"):
    """发送邮件

    参数 user：接收邮件的邮箱地址

    参数 text：邮件内容

    参数 theme：邮件主题

    参数 recNick：收件人昵称

    return Boolean类型
    """
    ret=True
    if not theme:
        theme=config.email['theme']
    if not recNick:
        recNick=config.email['recNick']
    try:
        msg=MIMEText(text,'plain','utf-8')
        msg['From']=formataddr([config.email['sendNick'],config.email['sender']]) 
        msg['To']=formataddr([recNick,user]) 
        msg['Subject']=theme

        server=smtplib.SMTP_SSL("smtp.qq.com", 465) 
        server.login(config.email['sender'], config.email['pwd']) 
        server.sendmail(config.email['sender'],[user,],msg.as_string())
        server.quit()
    except Exception:
        ret=False
    return ret
get_sysinfodesffafew=None
def get_sysinfo():
    """获取系统信息

    return dict类型
    """
    global get_sysinfodesffafew
    if get_sysinfodesffafew:
        sysinfo=get_sysinfodesffafew
    else:
        sysinfo={}
        sysinfo['platform']=platform.platform()        #获取操作系统名称及版本号，'Linux-3.13.0-46-generic-i686-with-Deepin-2014.2-trusty'  
        sysinfo['version']=platform.version()         #获取操作系统版本号，'#76-Ubuntu SMP Thu Feb 26 18:52:49 UTC 2015'
        sysinfo['architecture']=platform.architecture()    #获取操作系统的位数，('32bit', 'ELF')
        sysinfo['machine']=platform.machine()         #计算机类型，'i686'
        sysinfo['node']=platform.node()            #计算机的网络名称，'XF654'
        sysinfo['processor']=platform.processor()       #计算机处理器信息，''i686'
        sysinfo['uname']=platform.uname()           #包含上面所有的信息汇总，('Linux', 'XF654', '3.13.0-46-generic', '#76-Ubuntu SMP Thu Feb 26 18:52:49 UTC 2015', 'i686', 'i686')
        sysinfo['start_time']=times()
        get_sysinfodesffafew=sysinfo
            # 还可以获得计算机中python的一些信息：
            # import platform
            # platform.python_build()
            # platform.python_compiler()
            # platform.python_branch()
            # platform.python_implementation()
            # platform.python_revision()
            # platform.python_version()
            # platform.python_version_tuple()
    return sysinfo
def Template(path,**context):
    "模板渲染引擎函数,使用配置的模板路径"
    return Templates(str(config.app['tpl_folder'])+str(path),**context)
def Templates(path,**context):
    "模板渲染引擎函数，需要完整的模板目录文件"
    lookup = TemplateLookup(directories=[''])
    # body=''
    # with open(path, 'r',encoding='utf-8') as f:
    #     contents=f.read()
    #     t=kcwTemplate(contents,lookup=lookup,module_directory=config.cache['path']+"/Template")
    #     body=t.render(**context)

    # t=kcwTemplate(filename=path,module_directory=config.cache['path']+"/Template",lookup=lookup)
    t=lookup.get_template(path)
    body=t.render(**context)
    return body
def getfunction(strs):
    """获取指定文件对象
    
    strs :app.index.common.autoload  获取app/index/common/目录下的autoload对象
    """
    return importlib.import_module(strs)
def mysql(table=None,configss=None):
    """mysql数据库操作实例
    
    参数 table：表名

    参数 configss 数据库配置  可以传数据库名字符串
    """
    dbs=kcwmysql.mysql()
    if table is None:
        return dbs
    elif configss:
        return dbs.connect(configss).table(table)
    else:
        return dbs.connect(config.database).table(table)
def sqlite(table=None,configss=None):
    """sqlite数据库操作实例
    
    参数 table：表名

    参数 configss 数据库配置  可以传数据库名字符串
    """
    dbs=kcwsqlite.sqlite()
    if table is None:
        return dbs
    elif configss:
        return dbs.connect(configss).table(table)
    else:
        return dbs.connect(config.sqlite).table(table)
def M(table=None,confi=None):
    """数据库操作实例
    
    参数 table：表名

    参数 confi 数据库配置  可以传数据库名字符串
    """
    if confi:
        if confi['type']=='sqlite':
            return sqlite(table,confi)
        else:
            return mysql(table,confi)
    else:
        if config.database['type']=='sqlite':
            return sqlite(table)
        else:
            return mysql(table)
def mongo(table=None,configss=None):
    """mongodb数据库操作实例
    
    参数 table：表名(mongodb数据库集合名)

    参数 configss mongodb数据库配置  可以传数据库名字符串
    """
    mObj=kcwmongodb.mongo()
    if table is None:
        return mObj
    elif configss:
        return mObj.connect(configss).table(table)
    else:
        return mObj.connect(config.mongo).table(table)
def is_index(params,index):
    """判断列表或字典里的索引是否存在

    params  列表或字典

    index   索引值

    return Boolean类型
    """
    try:
        params[index]
    except KeyError:
        return False
    except IndexError:
        return False
    else:
        return True
def set_cache(name,values,expire="no"):
    """设置缓存

    参数 name：缓存名

    参数 values：缓存值

    参数 expire：缓存有效期 0表示永久  单位 秒
    
    return Boolean类型
    """
    return kcwcache.cache().set_cache(name,values,expire)
def get_cache(name):
    """获取缓存

    参数 name：缓存名

    return 或者的值
    """
    return kcwcache.cache().get_cache(name)
def del_cache(name):
    """删除缓存

    参数 name：缓存名

    return Boolean类型
    """
    return kcwcache.cache().del_cache(name)
def md5(strs):
    """md5加密
    
    参数 strs：要加密的字符串

    return String类型
    """
    m = hashlib.md5()
    b = strs.encode(encoding='utf-8')
    m.update(b)
    return m.hexdigest()
def times():
    """生成时间戳整数 精确到秒(10位数字)
    
    return int类型
    """
    return int(time.time())
def json_decode(strs):
    """json字符串转python类型"""
    try:
        return json.loads(strs)
    except Exception:
        return []
def json_encode(strs):
    """python列表或字典转成字符串"""
    try:
        return json.dumps(strs,ensure_ascii=False)
    except Exception:
        return ""
def dateoperator(date,years=0,formats='%Y%m%d%H%M%S',months=0, days=0, hours=0, minutes=0,seconds=0,
                 leapdays=0, weeks=0, microseconds=0,
                 year=None, month=None, day=None, weekday=None,
                 yearday=None, nlyearday=None,
                 hour=None, minute=None, second=None, microsecond=None):
    """日期相加减计算
    date 2019-10-10
    formats 设置需要返回的时间格式 默认%Y%m%d%H%M%S
    
    years 大于0表示加年  反之减年
    months 大于0表示加月  反之减月
    days 大于0表示加日  反之减日

    return %Y%m%d%H%M%S
    """
    formatss='%Y%m%d%H%M%S'
    date=re.sub('[-年/月:：日 时分秒]','',date)
    if len(date) < 8:
        return None
    if len(date) < 14:
        s=14-len(date)
        i=0
        while i < s:
            date=date+"0"
            i=i+1
    d = core_datetime.datetime.strptime(date, formatss)
    strs=(d + core_relativedelta(years=years,months=months, days=days, hours=hours, minutes=minutes,seconds=seconds,
                 leapdays=leapdays, weeks=weeks, microseconds=microseconds,
                 year=year, month=month, day=day, weekday=weekday,
                 yearday=yearday, nlyearday=nlyearday,
                 hour=hour, minute=minute, second=second, microsecond=microsecond))
    strs=strs.strftime(formats)
    return strs
def get_folder():
    '获取当前框架目录'
    return os.path.split(os.path.realpath(__file__))[0][:-7] #当前框架目录
# aa=[]
def get_file(folder='./',is_folder=True,suffix="*",lists=[],append=False):
    """获取文件夹下所有文件夹和文件

    folder 要获取的文件夹路径

    is_folder  是否返回列表中包含文件夹

    suffix 获取指定后缀名的文件 默认全部
    """
    if not append:
        lists=[]
    lis=os.listdir(folder)
    for files in lis:
        if not os.path.isfile(folder+"/"+files):
            if is_folder:
                zd={"type":"folder","path":folder+"/"+files,'name':files}
                lists.append(zd)
            get_file(folder+"/"+files,is_folder,suffix,lists,append=True)
        else:
            if suffix=='*':
                zd={"type":"file","path":folder+"/"+files,'name':files}
                lists.append(zd)
            else:
                if files[-(len(suffix)+1):]=='.'+str(suffix):
                    zd={"type":"file","path":folder+"/"+files,'name':files}
                    lists.append(zd)
    return lists

def list_to_tree(data, pk = 'id', pid = 'pid', child = 'lowerlist', root=0,childstatus=True):
    """列表转换tree
    
    data 要转换的列表

    pk 关联节点字段

    pid 父节点字段

    lowerlist 子节点列表

    root 主节点值

    childstatus 当子节点列表为空时是否需要显示子节点字段
    """
    arr = []
    for v in data:
        if v[pid] == root:
            kkkk=list_to_tree(data,pk,pid,child,v[pk],childstatus)
            if childstatus:
                v[child]=kkkk
            else:
                if kkkk:
                    v[child]=kkkk
            arr.append(v)
    return arr

        
class zip:
    def packzip(src,dst):
        "压缩"
        filelist = []
        if os.path.isfile(src):
            filelist.append(src)
        for root, dirs, files in os.walk(src):
            for name in files:
                filelist.append(os.path.join(root, name))
        zf = zipfile.ZipFile(dst, "w", zipfile.zlib.DEFLATED)
        for tar in filelist:
            arcname = tar[len(src):]
            zf.write(tar,arcname)
        zf.close()
    def unzip_file(dst, src):
        "解压"
        zf = zipfile.ZipFile(dst)
        zf.extractall(src)
        zf.close()
class kcwtar:
    def targz(src,dst):
        """
        打包目录为tar.gz
        :param src: 需要打包的目录
        :param dst: 压缩文件名
        :return: bool
        """
        with tarfile.open(dst, "w:gz") as tar:
            tar.add(src, arcname=os.path.basename(src))
        return True
    def untar(dst, src):
        """
        解压tar.gz文件
        :param dst: 压缩文件名
        :param src: 解压后的存放路径
        :return: bool
        """
        try:
            t = tarfile.open(dst)
            t.extractall(path = src)
            return True
        except Exception as e:
            return False

class response:
    tpldata={} #公共模板变量
    def tpl(path=None,status='200 ok',response_cache=False,ETag=None,header={"Content-Type":"text/html; charset=utf-8"},**context):
        """返回模板内容
        
        path 文件地址

        status 响应状态码

        response_cache 是否启用浏览器缓存  响应状态码200 ok时有效

        ETag 缓存标识  响应状态码200 ok时有效

        header 响应头
        """
        for k in dir(response):
            if k not in ['download','json','pic','redirect','tpl','tpldata','video'] and k[-2:]!='__':
                try:
                    context[k]=response.__dict__[k]
                except KeyError:
                    pass
        context['config']=config
        headers=copy.deepcopy(header)
        getroutecomponent=globals.VAR.component
        if path:
            if path[:1]=="/":
                Temppath=path
            else:
                Temppath="/"+getroutecomponent[1]+"/controller/"+getroutecomponent[2]+"/tpl/"+path+".html"
        else:
            Temppath="/"+getroutecomponent[1]+"/controller/"+getroutecomponent[2]+"/tpl/"+getroutecomponent[3]+"/"+getroutecomponent[4]+".html"
        if status=='200 ok' and response_cache:
            if not ETag:
                ttt=''
                for k in context.keys():
                    ttt+=k+str(context[k])
                ETag=md5(Temppath+ttt+globals.HEADER.URL)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if HTTP_IF_NONE_MATCH and get_cache(ETag):
                status="304 Not Modified"
                body=''
            else:
                if isinstance(response_cache,int) and response_cache>1:
                    headers['response_cache']=str(response_cache)+" s"
                    set_cache(ETag,1,response_cache)
                else:
                    headers['response_cache']="default"
                    set_cache(ETag,1)
                body=Template(Temppath,**context)
            dateArray = core_datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            headers['Last-Modified']=otherStyleTime
            headers['ETag']=ETag
            return body,status,headers
        elif status:
            return Template(Temppath,tpldata=response.tpldata,**context),status,headers
        else:
            return Template(Temppath,tpldata=response.tpldata,**context),'200 ok',headers
    def json(res=[],status='200 ok',response_cache=False,ETag=None,header={"Content-Type":"application/json; charset=utf-8","Access-Control-Allow-Origin":"*"}):
        """响应json内容

        res  body内容

        status 响应状态码

        response_cache 是否启用浏览器缓存  响应状态码200 ok时有效

        ETag 缓存标识  响应状态码200 ok时有效

        header 响应头
        """
        headers=copy.deepcopy(header)
        if status=='200 ok' and response_cache:
            if not ETag:
                ETag=md5(globals.HEADER.URL)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                status="304 Not Modified"
                body=''
            else:
                if isinstance(response_cache,int) and response_cache>1:
                    set_cache(ETag,1,response_cache)
                    headers['response_cache']=str(response_cache)+" s"
                else:
                    set_cache(ETag,1)
                    headers['response_cache']="default"
                body=json_encode(res)
            dateArray = core_datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            headers['Last-Modified']=otherStyleTime
            headers['ETag']=ETag
            
        else:
            body=json_encode(res)
        return body,status,headers
    def pic(body,response_cache=True,ETag=None):
        """输出图片
        
        body 图片二进制内容或图片路径 建议使用图片路径

        response_cache 是否启用浏览器缓存  body使用图片路径时有效

        ETag 缓存标识
        """
        status='200 ok'
        header={"Cache-Control":"public, max-age=2592000"}
        if isinstance(body,str):
            if response_cache:
                if not ETag:
                    ETag=md5(body+globals.HEADER.URL)
                try:
                    HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
                except:
                    HTTP_IF_NONE_MATCH=None
                if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                    status="304 Not Modified"
                    body=''
                else:
                    if isinstance(response_cache,int) and response_cache>1:
                        set_cache(ETag,1,response_cache)
                    else:
                        set_cache(ETag,1)
                    filename=body
                    f=open(filename,"rb")
                    body=f.read()
                    f.close()
                    kind = filetype.guess(filename)
                    try:
                        header['Content-Type']=kind.mime
                    except:
                        header['Content-Type']="image/png"
            else:
                filename=body
                f=open(filename,"rb")
                body=f.read()
                f.close()
                kind = filetype.guess(filename)
                try:
                    header['Content-Type']=kind.mime
                except:
                    header['Content-Type']="image/png"
            dateArray = core_datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            header['Last-Modified']=otherStyleTime
            header['ETag']=ETag
        else:
            header['Content-Type']="image/png"
        return body,status,header
    def video(body):
        """输出音频
        
        body 音频二进制内容或音频路径
        """
        status='200 ok'
        header={"Cache-Control":"public, max-age=2592000"}
        if isinstance(body,str):
            ETag=md5(body)
            try:
                HTTP_IF_NONE_MATCH=globals.HEADER.GET['HTTP_IF_NONE_MATCH']
            except:
                HTTP_IF_NONE_MATCH=None
            if(HTTP_IF_NONE_MATCH and get_cache(ETag)):
                status="304 Not Modified"
                body=''
            else:
                set_cache(ETag,1,2592000)
                filename=body
                f=open(filename,"rb")
                body=f.read()
                f.close()
                kind = filetype.guess(filename)
                try:
                    header['Content-Type']=kind.mime
                except:
                    header['Content-Type']="video/mp4"
            dateArray = core_datetime.datetime.utcfromtimestamp(times()-86400)
            otherStyleTime = dateArray.strftime('%a, %d %b %Y %H:%M:%S GMT')
            header['Last-Modified']=otherStyleTime
            header['ETag']=ETag
        else:
            header['Content-Type']="video/mp4"
        return body,status,header
    # def video(body):
    #     """输出音频
        
    #     body 音频二进制内容或音频路径
    #     """
    #     if isinstance(body,str):
    #         filename=body
    #         f=open(filename,"rb")
    #         body=f.read()
    #         f.close()
    #         kind = filetype.guess(filename)
    #         try:
    #             ContentType=kind.mime
    #         except:
    #            ContentType="video/mp4" 
    #     else:
    #         ContentType="video/mp4"
    #     return body,'200 ok',{"Content-Type":ContentType,"Cache-Control":"public, max-age=600"}
    def download(pathname):
        """下载文件
        
        pathname 文件路径
        """
        if os.path.isfile(pathname):
            f=open(pathname,"rb")
            body=f.read()
            f.close()
            kind = filetype.guess(pathname)
            try:
                return body,"200 ok",{"Content-Type":"application/"+kind.mime,"Accept-Ranges":"bytes"}
            except:
                return body,"200 ok",{"Content-Type":"application/text","Accept-Ranges":"bytes"}
        else:
            return Templates('E:\doc\python\kcwebplus\kcweb/tpl/err.html',title="文件不存在",content="文件不存在",imgsrc=config.domain['kcwebimg']+"/icon/error.png",config=config)
    def redirect(url,status="302 Found",html='',header={"Content-Type":"application/html; charset=utf-8"}):
        """重定向

        参数 url 重定向地址 必须

        参数 status 响应码  可选

        参数 html body响应内容 可选

        参数 header 响应头  可选
        """
        header['Location']=url
        return html,status,header
class create:
    appname=None
    modular=None
    path=get_folder() #当前框架目录
    def __init__(self,appname="app",modular="api"):
        self.appname=str(appname)
        self.modular=str(modular)
    def uninstallplug(self,plug):
        """卸载插件

        plug 插件名
        """
        f=open(self.appname+"/"+self.modular+"/controller/__init__.py","r",encoding='utf-8')
        text=f.read()
        f.close()
        text=re.sub("\nfrom . import "+plug,"",text)
        text=re.sub("from . import "+plug,"",text)
        f=open(self.appname+"/"+self.modular+"/controller/__init__.py","w",encoding='utf-8')
        f.write(text)
        f.close()
        shutil.rmtree(self.appname+"/"+self.modular+"/controller/"+plug)
        return True,"成功"
    def packplug(self,plug):
        """打包插件
        
        plug 插件名
        """
        """打包模块"""
        if os.path.exists(self.appname+"/"+self.modular+"/controller/"+plug):
            zip.packzip(self.appname+"/"+self.modular+"/controller/"+plug,self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return True,"成功"
        else:
            return False,"失败"
    def uploadplug(self,plug,username='',password='',cli=False):
        "上传一个插件"
        if not os.path.isfile(self.appname+"/"+self.modular+"/controller/"+plug+".zip"):
            self.packplug(plug=plug)
        i=0
        http=Http()
        http.set_timeout=300
        while True:
            timestamp=times()
            sign=md5(str(username)+str(timestamp)+md5(md5(password)))
            # http.set_header['username']=username
            # http.set_header['timestamp']=str(timestamp)
            # http.set_header['sign']=sign
            http.openurl(config.domain['kcwebapi']+"/user/userinfo/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign)
            arr=json_decode(http.get_text)
            if not arr:
                os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                if config.app['app_debug']:
                    print(http.get_text)
                return False,"用户身份验证失败，服务器暂时无法处理"
            if (arr['code']==-1 or arr['code']==2) and cli:
                if i >= 3:
                    os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                    return False,"用户名或密码错误"
                elif i:
                    print("用户名或密码错误，请重新输入")
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                else:
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                i+=1
            elif arr['code']==0:
                break
            else:
                os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                return False,arr['msg']
        http.openurl(config.domain['kcwebapi']+"/user/uploadplug/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,'POST',
        data={'name':str(plug),'describes':'','modular':self.modular},
        files={'file':open(self.appname+"/"+self.modular+"/controller/"+plug+".zip", 'rb')})
        arr=json_decode(http.get_text)
        if not arr:
            os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            if config.app['app_debug']:
                print(http.get_text)
            return False,"上传失败，服务器暂时无法处理上传"
        elif arr['code']==-1 or arr['code']==2:
            os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,"用户名或密码错误"
        elif arr['code']==0:
            os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return True,arr['msg']
        elif arr['code']==0:
            os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,arr['msg']
        else:
            os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
            return False,arr['msg']
    def installplug(self,plug,edition='',token='',cli=False,mandatory=False):
        """创建一个插件，如果您的模块目录下没有插件包，则创建默认插件文件
        
        plug 插件名
        """
        plug=str(plug)
        if os.path.exists(self.appname+"/"+self.modular+"/controller/"+plug) and not mandatory:
            return False,"该插件已存在"
        else:
            http=Http()
            i=0
            j=0
            tplug=plug
            modular=self.modular
            while True:
                http.openurl(config.domain['kcwebapi']+"/pub/plug","GET",params={"modular":modular,"name":str(tplug),"edition":str(edition),"token":token})
                arr=json_decode(http.get_text)
                if arr:
                    if arr['code']==-1 and cli:
                        if i >= 3:
                            return False,plug+"插件授权码错误"
                        elif i:
                            token = input("授权码错误，请重新输入授权码，从而获得该插件\n")
                        else:
                            token = input("请输入授权码，从而获得该插件\n")
                        i+=1
                    elif arr['code']==-1:
                        return False,plug+"插件授权码错误"
                    elif not arr['data']:
                        modular="api"
                        tplug="index" #默认插件
                    elif arr['code']==0 and arr['data']:
                        i=0
                        j+=1
                        arr=arr['data']
                        r=requests.get(arr['dowurl'],verify=False)
                        f = open(self.appname+"/"+self.modular+"/controller/"+plug+".zip", "wb")
                        for chunk in r.iter_content(chunk_size=512):
                            if chunk:
                                f.write(chunk)
                        f.close()
                        if zipfile.is_zipfile(self.appname+"/"+self.modular+"/controller/"+plug+".zip") and os.path.isfile(self.appname+"/"+self.modular+"/controller/"+plug+".zip"):
                            break
                        if j >= 10:
                            return False,str(plug)+"插件下载失败"
                        time.sleep(0.1)
                    else:
                        return False,str(plug)+"插件搜索失败"
                else:
                    return False,self.modular+"模块下找不到"+str(plug)+"插件"
            if os.path.isfile(self.appname+"/"+self.modular+"/controller/"+plug+".zip"):#安装打包好的插件
                zip.unzip_file(self.appname+"/"+self.modular+"/controller/"+plug+".zip",self.appname+"/"+self.modular+"/controller/"+plug+"/")
                os.remove(self.appname+"/"+self.modular+"/controller/"+plug+".zip")
                if os.path.isfile(self.appname+"/"+self.modular+"/controller/"+plug+"/install.txt"): #安装依赖包
                    install_requires=[]
                    try:
                        f=open(self.appname+"/"+self.modular+"/controller/"+plug+"/install.txt")
                        while True:
                            line = f.readline()
                            if not line:
                                break
                            elif len(line) > 2:
                                install_requires.append(line)
                        f.close()
                    except:
                        shutil.rmtree(self.appname+"."+self.modular+"/controller/"+plug)
                        return False,"error"
                    if len(install_requires):
                        try:
                            install_requires.insert(0,"install")
                            if 0 != pip.main(install_requires):
                                shutil.rmtree(self.appname+"/"+self.modular+"/controller/"+plug)
                                return False,"error"
                        except AttributeError as e:
                            shutil.rmtree(self.appname+"/"+self.modular+"/controller/"+plug)
                            if config.app['app_debug']:
                                print("建议更新您的pip版本。参考命令：Python -m pip install --user --upgrade pip -i https://mirrors.aliyun.com/pypi/simple/")
                            return False,str(e)
                if os.path.isfile(self.appname+"."+self.modular+"/controller/"+plug+"/install.py"):
                    try:
                        m=importlib.import_module(self.appname+"."+self.modular+"/controller/"+plug+".install")
                    except:
                        shutil.rmtree(self.appname+"."+self.modular+"/controller/"+plug)
                        print(traceback.format_exc())
                        return False,"插件依赖包文件不存在或依赖包文件格式错误"
                    else:
                        try:
                            a=m.install()
                        except:
                            shutil.rmtree(self.appname+"."+self.modular+"/controller/"+plug)
                            return False,"插件依赖包install函数被破坏"
                        # if not a[0]:
                        #     shutil.rmtree(self.appname+"."+self.modular+"/controller/"+plug)
                        #     return False,str(a[1])

                f=open(self.appname+"/"+self.modular+"/controller/__init__.py","r",encoding='utf-8')
                text=f.read()
                f.close()
                text=re.sub("\nfrom . import "+plug,"",text)
                text=re.sub("from . import "+plug,"",text)
                f=open(self.appname+"/"+self.modular+"/controller/__init__.py","w",encoding='utf-8')
                text+="\nfrom . import "+plug
                f.write(text)
                f.close()

                f=open(self.appname+"/"+self.modular+"/controller/"+plug+"/common/autoload.py","r",encoding='utf-8')
                text=f.read()
                f.close()
                text=re.sub("app.api",self.appname+"."+self.modular,text)
                f=open(self.appname+"/"+self.modular+"/controller/"+plug+"/common/autoload.py","w",encoding='utf-8')
                f.write(text)
                f.close()

                
                return True,"插件安装成功，"+plug+"=="+str(arr['edition'])
            else:
                return False,str(plug)+"插件获取失败"
    def uninstallmodular(self):
        "卸载模块"
        f=open(self.appname+"/__init__.py","r")
        text=f.read()
        f.close()
        text=re.sub("\nfrom . import "+self.modular,"",text)
        text=re.sub("from . import "+self.modular,"",text)
        f=open(self.appname+"/__init__.py","w")
        f.write(text)
        f.close()
        shutil.rmtree(self.appname+"/"+self.modular)
        return True,"成功"
    def packmodular(self):
        """打包模块"""
        if os.path.exists(self.appname+"/"+self.modular):
            zip.packzip(self.appname+"/"+self.modular,self.appname+"/"+self.modular+".zip")
            return True,"成功"
        else:
            return False,"失败"
    def uploadmodular(self,username='',password='',cli=False):
        "上传模块"
        if not os.path.isfile(self.appname+"/"+self.modular+".zip"):
            self.packmodular()
        i=0
        http=Http()
        http.set_timeout=300
        while True:
            timestamp=times()
            sign=md5(str(username)+str(timestamp)+md5(md5(password)))
            # http.set_header['username']=username
            # http.set_header['timestamp']=str(timestamp)
            # http.set_header['sign']=sign
            http.openurl(config.domain['kcwebapi']+"/user/uploadmodular/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,'POST',
            data={'name':str(self.modular),'describes':''},
            files={'file':open(self.appname+"/"+self.modular+".zip", 'rb')})
            arr=json_decode(http.get_text)
            if not arr:
                os.remove(self.appname+"/"+self.modular+".zip")
                if config.app['app_debug']:
                    print(http.get_text)
                return False,"用户身份验证失败，服务器暂时无法处理"
            if (arr['code']==-1 or arr['code']==2) and cli:
                if i >= 3:
                    os.remove(self.appname+"/"+self.modular+".zip")
                    return False,"用户名或密码错误"
                elif i:
                    print("用户名或密码错误，请重新输入")
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                else:
                    username = input("请输入用户名（手机号）\n")
                    password = input("请输入密码\n")
                i+=1
            elif arr['code']==0:
                break
            elif arr['code']==-1:
                os.remove(self.appname+"/"+self.modular+".zip")
                return False,"用户名或密码错误"
            else:
                os.remove(self.appname+"/"+self.modular+".zip")
                return False,arr['msg']
        
        http.openurl(config.domain['kcwebapi']+"/user/uploadmodular/?username="+username+"&timestamp="+str(timestamp)+"&sign="+sign,'POST',
        data={'name':str(self.modular),'describes':''},
        files={'file':open(self.appname+"/"+self.modular+".zip", 'rb')})
        arr=json_decode(http.get_text)
        if not arr:
            os.remove(self.appname+"/"+self.modular+".zip")
            if config.app['app_debug']:
                print(http.get_text)
            return False,"上传失败，服务器暂时无法处理上传"
        elif arr['code']==-1 or arr['code']==2:
            os.remove(self.appname+"/"+self.modular+".zip")
            return False,"用户名或密码错误"
        elif arr['code']==0:
            os.remove(self.appname+"/"+self.modular+".zip")
            return True,arr['msg']
        else:
            os.remove(self.appname+"/"+self.modular+".zip")
            return False,arr['msg']
    def installmodular(self,token='',cli=False):
        "创建模块，如果应用不存，则创建默认应用，如果在您的应用目录下没有模块包，则创建默认模块文件"
        if not os.path.exists(self.appname):
            # os.makedirs(self.appname)
            r=requests.get(config.domain['kcwebfile']+"/kcweb/app.zip")
            f = open("./app.zip", "wb")
            for chunk in r.iter_content(chunk_size=512):
                if chunk:
                    f.write(chunk)
            f.close()
            zip.unzip_file("./app.zip","./"+self.appname)
            os.remove("./app.zip")
            if not os.path.isfile("./server.py"):
                if "Windows" in platform.platform():
                    pythonname="python"
                else:
                    pythonname="python3"
                servertext=('# -*- coding: utf-8 -*-\n #gunicorn -b 0.0.0.0:39010 '+self.appname+':app\n'+
                        'from kcweb import web\n'+
                        'import '+self.appname+' as application\n'+
                        'app=web(__name__,application)\n'+
                        'if __name__ == "__main__":\n'+
                        '    #host监听ip port端口 name python解释器名字 (windows一般是python  linux一般是python3)\n'+
                        '    app.run(host="0.0.0.0",port="39001",name="'+pythonname+'")')
                f=open("./server.py","w+",encoding='utf-8')
                f.write(servertext)
                f.close()
            f=open(self.appname+"/common/autoload.py","w",encoding='utf-8')
            f.write("from kcweb.common import *\n"+
                    "from "+self.appname+" import config\n"+
                    "G=globals.G")
            f.close()

            content=''
            f=open(self.appname+"/"+self.modular+"/common/autoload.py","r",encoding='utf-8')
            while True:
                line = f.readline()
                if not line:
                    break
                elif 'from' not in line and 'import' not in line:
                    content+=line
            f.close()
            f=open(self.appname+"/"+self.modular+"/common/autoload.py","w",encoding='utf-8')
            f.write("from "+self.appname+".common import *\n"+content)
            f.close()

            content=''
            f=open(self.appname+"/"+self.modular+"/controller/index/common/autoload.py","r",encoding='utf-8')
            while True:
                line = f.readline()
                if not line:
                    break
                elif 'from' not in line and 'import' not in line:
                    content+=line
            f.close()
            f=open(self.appname+"/"+self.modular+"/controller/index/common/autoload.py","w",encoding='utf-8')
            f.write("from "+self.appname+"."+self.modular+".common import *\n"+content)
            f.close()
            return True,"应用创建成功"
        else:
            if not os.path.isfile(self.appname+"/__init__.py") or not os.path.exists(self.appname+"/common"):
                return False,self.appname+"不是kcweb应用"
        if os.path.exists(self.appname+"/"+self.modular):
            return False,self.appname+"/"+self.modular+"已存在"
        else:
            http=Http()
            i=0
            modular=self.modular
            while True:
                http.openurl(config.domain['kcwebapi']+"/pub/modular","POST",params={"name":modular,"token":token})
                arr=json_decode(http.get_text)
                if arr:
                    if arr['code']==-1 and cli:
                        if i >= 3:
                            return False,self.modular+"模块授权码错误"
                        elif i:
                            token = input("授权码错误，请重新输入授权码，从而获得该模块\n")
                        else:
                            token = input("请输入授权码，从而获得该模块\n")
                        i+=1
                    elif arr['code']==-1:
                        return False,self.modular+"模块授权码错误"
                    elif not arr['data']:
                        modular="api"
                    elif arr['code']==0 and arr['data']:
                        arr=arr['data']
                        #循环下载模块
                        i=0
                        while i < 5:
                            r=requests.get(arr['dowurl'])
                            f = open(self.appname+"/"+self.modular+".zip", "wb")
                            for chunk in r.iter_content(chunk_size=1024*100):
                                if chunk:
                                    f.write(chunk)
                            f.close()
                            time.sleep(0.3)
                            if os.path.isfile(self.appname+"/"+self.modular+".zip"):
                                break
                            i+=1
                        if os.path.isfile(self.appname+"/"+self.modular+".zip"):#安装打包好的模块
                            zip.unzip_file(self.appname+"/"+self.modular+".zip",self.appname+"/"+self.modular+"/")
                            os.remove(self.appname+"/"+self.modular+".zip")

                            if os.path.isfile(self.appname+"/"+self.modular+"/install.txt"): #安装依赖包
                                install_requires=[]
                                try:
                                    f=open(self.appname+"/"+self.modular+"/install.txt")
                                    while True:
                                        line = f.readline()
                                        if not line:
                                            break
                                        elif len(line) > 3:
                                            install_requires.append(line)
                                    f.close()
                                except:
                                    shutil.rmtree(self.appname+"/"+self.modular)
                                    return False,"error"
                                if len(install_requires):
                                    try:
                                        install_requires.insert(0,"install")
                                        if 0 != pip.main(install_requires):
                                            shutil.rmtree(self.appname+"/"+self.modular)
                                            return False,"error"
                                    except AttributeError as e:
                                        shutil.rmtree(self.appname+"/"+self.modular)
                                        if config.app['app_debug']:
                                            print("建议更新您的pip版本。参考命令：Python -m pip install --user --upgrade pip -i https://mirrors.aliyun.com/pypi/simple/")
                                        return False,str(e)
                            if os.path.isfile(self.appname+"/"+self.modular+"/install.py"):#如果存在依赖文件
                                try:
                                    m=importlib.import_module(self.appname+'.'+self.modular+'.install')
                                except:
                                    shutil.rmtree(self.appname+"/"+self.modular)
                                    print(traceback.format_exc())
                                    return False,"模块依赖包文件不存在或依赖包文件格式错误"
                                else:
                                    try:
                                        a=m.install()
                                    except:
                                        shutil.rmtree(self.appname+"/"+self.modular)
                                        return False,"模块依赖包install方法被破坏"
                                    # if not a[0]:
                                    #     shutil.rmtree(self.appname+"/"+self.modular)
                                    #     return False,str(a[1])
                            content="\nfrom . import "+self.modular
                            f=open(self.appname+"/__init__.py","a",encoding='utf-8')
                            f.write(content)
                            f.close()

                            content=''
                            f=open(self.appname+"/"+self.modular+"/common/autoload.py","r",encoding='utf-8')
                            while True:
                                line = f.readline()
                                if not line:
                                    break
                                elif 'from' not in line and 'import' not in line:
                                    content+=line
                            f.close()
                            f=open(self.appname+"/"+self.modular+"/common/autoload.py","w",encoding='utf-8')
                            f.write("from "+self.appname+".common import *\n"+content)
                            f.close()

                            content=''
                            f=open(self.appname+"/"+self.modular+"/controller/index/common/autoload.py","r",encoding='utf-8')
                            while True:
                                line = f.readline()
                                if not line:
                                    break
                                elif 'from' not in line and 'import' not in line:
                                    content+=line
                            f.close()
                            f=open(self.appname+"/"+self.modular+"/controller/index/common/autoload.py","w",encoding='utf-8')
                            f.write("from "+self.appname+"."+self.modular+".common import *\n"+content)
                            f.close()
                        else:
                            return False,self.modular+"模块下载失败"
                        if not os.path.isfile("./server.py"):
                            if "Windows" in platform.platform():
                                pythonname="python"
                            else:
                                pythonname="python3"
                            # sys.argv[0]=re.sub('.py','',sys.argv[0])
                            servertext=('# -*- coding: utf-8 -*-\n#gunicorn -b 0.0.0.0:39010 '+self.appname+':app\n'+
                                    'from kcweb import web\n'+
                                    'import '+self.appname+' as application\n'+
                                    'app=web(__name__,application)\n'+
                                    'if __name__ == "__main__":\n'+
                                    '    #host监听ip port端口 name python解释器名字 (windows一般是python  linux一般是python3)\n'+
                                    '    app.run(host="0.0.0.0",port="39001",name="'+pythonname+'")')
                            f=open("./server.py","w+",encoding='utf-8')
                            f.write(servertext)
                            f.close()
                        return True,"安装成功"
                    else:

                        return False,"模块下载失败"
                else:
                    return False,"找不到"+self.modular+"模块"
    # def __zxmodular(self,sourcep): 
    #     "处理模块文件"
    #     path1=self.path+"/application/api"+sourcep
    #     path2=self.appname+"/"+self.modular+sourcep
    #     lists=os.listdir(path1)
    #     for files in lists:
    #         if os.path.isfile(path1+"/"+files):
    #             if ".py" in files:
    #                 content=Templates(path1+"/"+files,appname=self.appname,modular=self.modular)
    #                 f=open(path2+"/"+files,"w+",encoding='utf-8')
    #                 f.write(content)
    #                 f.close()
    #             else:
    #                 f=open(path1+"/"+files,"r",encoding='utf-8')
    #                 content=f.read()
    #                 f.close()
    #                 f=open(path2+"/"+files,"w+",encoding='utf-8')
    #                 f.write(content)
    #                 f.close()
    #         elif files != '__pycache__':
    #             if not os.path.exists(path2+"/"+files):
    #                 os.makedirs(path2+"/"+files)
    #             self.__zxmodular(sourcep+"/"+files)


# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .base import Type


class Epub(Type):
    """
    Implements the EPUB archive type matcher.
    """
    MIME = 'application/epub+zip'
    EXTENSION = 'epub'

    def __init__(self):
        super(Epub, self).__init__(
            mime=Epub.MIME,
            extension=Epub.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 57 and
                buf[0] == 0x50 and buf[1] == 0x4B and
                buf[2] == 0x3 and buf[3] == 0x4 and
                buf[30] == 0x6D and buf[31] == 0x69 and
                buf[32] == 0x6D and buf[33] == 0x65 and
                buf[34] == 0x74 and buf[35] == 0x79 and
                buf[36] == 0x70 and buf[37] == 0x65 and
                buf[38] == 0x61 and buf[39] == 0x70 and
                buf[40] == 0x70 and buf[41] == 0x6C and
                buf[42] == 0x69 and buf[43] == 0x63 and
                buf[44] == 0x61 and buf[45] == 0x74 and
                buf[46] == 0x69 and buf[47] == 0x6F and
                buf[48] == 0x6E and buf[49] == 0x2F and
                buf[50] == 0x65 and buf[51] == 0x70 and
                buf[52] == 0x75 and buf[53] == 0x62 and
                buf[54] == 0x2B and buf[55] == 0x7A and
                buf[56] == 0x69 and buf[57] == 0x70)


class Zip(Type):
    """
    Implements the Zip archive type matcher.
    """
    MIME = 'application/zip'
    EXTENSION = 'zip'

    def __init__(self):
        super(Zip, self).__init__(
            mime=Zip.MIME,
            extension=Zip.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x50 and buf[1] == 0x4B and
                (buf[2] == 0x3 or buf[2] == 0x5 or
                    buf[2] == 0x7) and
                (buf[3] == 0x4 or buf[3] == 0x6 or
                    buf[3] == 0x8))


class Tar(Type):
    """
    Implements the Tar archive type matcher.
    """
    MIME = 'application/x-tar'
    EXTENSION = 'tar'

    def __init__(self):
        super(Tar, self).__init__(
            mime=Tar.MIME,
            extension=Tar.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 261 and
                buf[257] == 0x75 and
                buf[258] == 0x73 and
                buf[259] == 0x74 and
                buf[260] == 0x61 and
                buf[261] == 0x72)


class Rar(Type):
    """
    Implements the RAR archive type matcher.
    """
    MIME = 'application/x-rar-compressed'
    EXTENSION = 'rar'

    def __init__(self):
        super(Rar, self).__init__(
            mime=Rar.MIME,
            extension=Rar.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 6 and
                buf[0] == 0x52 and
                buf[1] == 0x61 and
                buf[2] == 0x72 and
                buf[3] == 0x21 and
                buf[4] == 0x1A and
                buf[5] == 0x7 and
                (buf[6] == 0x0 or
                    buf[6] == 0x1))


class Gz(Type):
    """
    Implements the GZ archive type matcher.
    """
    MIME = 'application/gzip'
    EXTENSION = 'gz'

    def __init__(self):
        super(Gz, self).__init__(
            mime=Gz.MIME,
            extension=Gz.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 2 and
                buf[0] == 0x1F and
                buf[1] == 0x8B and
                buf[2] == 0x8)


class Bz2(Type):
    """
    Implements the BZ2 archive type matcher.
    """
    MIME = 'application/x-bzip2'
    EXTENSION = 'bz2'

    def __init__(self):
        super(Bz2, self).__init__(
            mime=Bz2.MIME,
            extension=Bz2.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 2 and
                buf[0] == 0x42 and
                buf[1] == 0x5A and
                buf[2] == 0x68)


class SevenZ(Type):
    """
    Implements the SevenZ (7z) archive type matcher.
    """
    MIME = 'application/x-7z-compressed'
    EXTENSION = '7z'

    def __init__(self):
        super(SevenZ, self).__init__(
            mime=SevenZ.MIME,
            extension=SevenZ.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 5 and
                buf[0] == 0x37 and
                buf[1] == 0x7A and
                buf[2] == 0xBC and
                buf[3] == 0xAF and
                buf[4] == 0x27 and
                buf[5] == 0x1C)


class Pdf(Type):
    """
    Implements the PDF archive type matcher.
    """
    MIME = 'application/pdf'
    EXTENSION = 'pdf'

    def __init__(self):
        super(Pdf, self).__init__(
            mime=Pdf.MIME,
            extension=Pdf.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x25 and
                buf[1] == 0x50 and
                buf[2] == 0x44 and
                buf[3] == 0x46)


class Exe(Type):
    """
    Implements the EXE archive type matcher.
    """
    MIME = 'application/x-msdownload'
    EXTENSION = 'exe'

    def __init__(self):
        super(Exe, self).__init__(
            mime=Exe.MIME,
            extension=Exe.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 1 and
                buf[0] == 0x4D and
                buf[1] == 0x5A)


class Swf(Type):
    """
    Implements the SWF archive type matcher.
    """
    MIME = 'application/x-shockwave-flash'
    EXTENSION = 'swf'

    def __init__(self):
        super(Swf, self).__init__(
            mime=Swf.MIME,
            extension=Swf.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 2 and
                (buf[0] == 0x43 or
                    buf[0] == 0x46) and
                buf[1] == 0x57 and
                buf[2] == 0x53)


class Rtf(Type):
    """
    Implements the RTF archive type matcher.
    """
    MIME = 'application/rtf'
    EXTENSION = 'rtf'

    def __init__(self):
        super(Rtf, self).__init__(
            mime=Rtf.MIME,
            extension=Rtf.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 4 and
                buf[0] == 0x7B and
                buf[1] == 0x5C and
                buf[2] == 0x72 and
                buf[3] == 0x74 and
                buf[4] == 0x66)


class Nes(Type):
    """
    Implements the NES archive type matcher.
    """
    MIME = 'application/x-nintendo-nes-rom'
    EXTENSION = 'nes'

    def __init__(self):
        super(Nes, self).__init__(
            mime=Nes.MIME,
            extension=Nes.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x4E and
                buf[1] == 0x45 and
                buf[2] == 0x53 and
                buf[3] == 0x1A)


class Crx(Type):
    """
    Implements the CRX archive type matcher.
    """
    MIME = 'application/x-google-chrome-extension'
    EXTENSION = 'crx'

    def __init__(self):
        super(Crx, self).__init__(
            mime=Crx.MIME,
            extension=Crx.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x43 and
                buf[1] == 0x72 and
                buf[2] == 0x32 and
                buf[3] == 0x34)


class Cab(Type):
    """
    Implements the CAB archive type matcher.
    """
    MIME = 'application/vnd.ms-cab-compressed'
    EXTENSION = 'cab'

    def __init__(self):
        super(Cab, self).__init__(
            mime=Cab.MIME,
            extension=Cab.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                ((buf[0] == 0x4D and
                    buf[1] == 0x53 and
                    buf[2] == 0x43 and
                    buf[3] == 0x46) or
                    (buf[0] == 0x49 and
                        buf[1] == 0x53 and
                        buf[2] == 0x63 and
                        buf[3] == 0x28)))


class Eot(Type):
    """
    Implements the EOT archive type matcher.
    """
    MIME = 'application/octet-stream'
    EXTENSION = 'eot'

    def __init__(self):
        super(Eot, self).__init__(
            mime=Eot.MIME,
            extension=Eot.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 35 and
                buf[34] == 0x4C and
                buf[35] == 0x50 and
                ((buf[8] == 0x02 and
                    buf[9] == 0x00 and
                    buf[10] == 0x01) or
                (buf[8] == 0x01 and
                    buf[9] == 0x00 and
                    buf[10] == 0x00) or
                    (buf[8] == 0x02 and
                        buf[9] == 0x00 and
                        buf[10] == 0x02)))


class Ps(Type):
    """
    Implements the PS archive type matcher.
    """
    MIME = 'application/postscript'
    EXTENSION = 'ps'

    def __init__(self):
        super(Ps, self).__init__(
            mime=Ps.MIME,
            extension=Ps.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 1 and
                buf[0] == 0x25 and
                buf[1] == 0x21)


class Xz(Type):
    """
    Implements the XS archive type matcher.
    """
    MIME = 'application/x-xz'
    EXTENSION = 'xz'

    def __init__(self):
        super(Xz, self).__init__(
            mime=Xz.MIME,
            extension=Xz.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 5 and
                buf[0] == 0xFD and
                buf[1] == 0x37 and
                buf[2] == 0x7A and
                buf[3] == 0x58 and
                buf[4] == 0x5A and
                buf[5] == 0x00)


class Sqlite(Type):
    """
    Implements the Sqlite DB archive type matcher.
    """
    MIME = 'application/x-sqlite3'
    EXTENSION = 'sqlite'

    def __init__(self):
        super(Sqlite, self).__init__(
            mime=Sqlite.MIME,
            extension=Sqlite.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x53 and
                buf[1] == 0x51 and
                buf[2] == 0x4C and
                buf[3] == 0x69)


class Deb(Type):
    """
    Implements the DEB archive type matcher.
    """
    MIME = 'application/x-deb'
    EXTENSION = 'deb'

    def __init__(self):
        super(Deb, self).__init__(
            mime=Deb.MIME,
            extension=Deb.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 20 and
                buf[0] == 0x21 and
                buf[1] == 0x3C and
                buf[2] == 0x61 and
                buf[3] == 0x72 and
                buf[4] == 0x63 and
                buf[5] == 0x68 and
                buf[6] == 0x3E and
                buf[7] == 0x0A and
                buf[8] == 0x64 and
                buf[9] == 0x65 and
                buf[10] == 0x62 and
                buf[11] == 0x69 and
                buf[12] == 0x61 and
                buf[13] == 0x6E and
                buf[14] == 0x2D and
                buf[15] == 0x62 and
                buf[16] == 0x69 and
                buf[17] == 0x6E and
                buf[18] == 0x61 and
                buf[19] == 0x72 and
                buf[20] == 0x79)


class Ar(Type):
    """
    Implements the AR archive type matcher.
    """
    MIME = 'application/x-unix-archive'
    EXTENSION = 'ar'

    def __init__(self):
        super(Ar, self).__init__(
            mime=Ar.MIME,
            extension=Ar.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 6 and
                buf[0] == 0x21 and
                buf[1] == 0x3C and
                buf[2] == 0x61 and
                buf[3] == 0x72 and
                buf[4] == 0x63 and
                buf[5] == 0x68 and
                buf[6] == 0x3E)


class Z(Type):
    """
    Implements the Z archive type matcher.
    """
    MIME = 'application/x-compress'
    EXTENSION = 'Z'

    def __init__(self):
        super(Z, self).__init__(
            mime=Z.MIME,
            extension=Z.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 1 and
                ((buf[0] == 0x1F and
                    buf[1] == 0xA0) or
                (buf[0] == 0x1F and
                    buf[1] == 0x9D)))


class Lz(Type):
    """
    Implements the Lz archive type matcher.
    """
    MIME = 'application/x-lzip'
    EXTENSION = 'lz'

    def __init__(self):
        super(Lz, self).__init__(
            mime=Lz.MIME,
            extension=Lz.EXTENSION
        )

    def match(self, buf):
        return (len(buf) > 3 and
                buf[0] == 0x4C and
                buf[1] == 0x5A and
                buf[2] == 0x49 and
                buf[3] == 0x50)



# -*- coding: utf-8 -*-
import pymongo,re
from bson.objectid import ObjectId
from kcweb.config import mongo as mongodb
class mongo:
    __config=mongodb
    __clientobj=None
    __dbobj=None
    __tabobj=None
    def __setconn(self):
        if not self.__clientobj:
            if self.__config['retryWrites']:
                strs='mongodb://'+self.__config['host']+':'+self.__config['port']+'/'
            else:
                strs='mongodb://'+self.__config['host']+':'+self.__config['port']+'/?retryWrites=false'
            self.__clientobj = pymongo.MongoClient(strs)
        self.__dbobj = self.__clientobj[self.__config['db']]
        if self.__config['user'] and self.__config['password']:
            self.__dbobj.authenticate(self.__config['user'],self.__config['password']) #账号密码认证
        self.__tabobj=self.__dbobj[self.__table]
    def connect(self,config):
        """设置mongo链接信息 

        参数 config 参考配置信息格式  可以设置数据库名（以字符串形式）

        返回 mongodb对象
        """
        if config:
            if isinstance(config,dict):
                if "host" in config:
                    self.__config['host']=config['host']
                if "port" in config:
                    self.__config['port']=config['port']
                if "user" in config:
                    self.__config['user']=config['user']
                if "password" in config:
                    self.__config['password']=config['password']
                if "db" in config:
                    self.__config['db']=config['db']
            elif isinstance(config,str):
                self.__config['db']=config
            else:
                print("config类型错误,设置连接不生效")
        return self
    def getobj(self):
        "获取mongodb链接实例"
        self.__setconn()
        return self.__tabobj
    __order=None
    def order(self,order):
        """设置排序
        
        order [('_id', -1)]
        """
        self.__order=order
        return self
    def select(self,id=None):
        """查询所有文档

        返回 文档列表
        """
        
        self.__setconn()
        if id:
            self.where('_id',id)
        lists=[]
        if self.__field:
            arr=self.__tabobj.find(self.__where,self.__field,sort=self.__order)
        else:
            arr=self.__tabobj.find(self.__where,sort=self.__order)
        print(self.__order)
        if self.__limit:
            if self.__limit[1]:
                arr.limit(self.__limit[1])
                arr.skip(self.__limit[0])
            else:
                arr.limit(self.__limit[0])
        for k in arr:
            try: k['_id']
            except: pass
            else: k['_id']=str(k['_id'])
            lists.append(k)
        self.__None()
        return lists
    def find(self,id=None):
        """查询一条文档
        
        返回 文档内容
        """
        self.__setconn()
        if id:
            self.where('_id',id)
        if self.__field:
            arr = self.__tabobj.find_one(self.__where,self.__field)
        else:
            arr = self.__tabobj.find_one(self.__where)
        try: arr['_id']
        except: pass
        else: arr['_id']=str(arr['_id'])
        self.__None()
        return arr
    def countlist(self):
        """查询文档数量和所有文档
        
        返回 文档数量,文档列表
        """
        self.__setconn()
        lists=[]
        if self.__field:
            arr=self.__tabobj.find(self.__where,self.__field)
        else:
            arr=self.__tabobj.find(self.__where)
        if self.__limit:
            if self.__limit[1]:
                arr.limit(self.__limit[1])
                arr.skip(self.__limit[0])
            else:
                arr.limit(self.__limit[0])
        for k in arr:
            try: k['_id']
            except: pass
            else: k['_id']=str(k['_id'])
            lists.append(k)
        self.__None()
        return arr.count(),lists
    def count(self):
        """查询文档数量
        
        返回 文档数量
        """
        self.__setconn()
        count=self.__tabobj.find(self.__where,{}).count()
        self.__None()
        return count
    def update(self,data,multi=True):
        """文档更新
         
        参数 data 要更新的内容  格式：{"name":"测试","age":20}

        multi 默认True  是否全部更新
        """
        #{ "count" : { $gt : 3 } } , { $set : { "test2" : "OK"} }
        self.__setconn()
        # print(self.__where)
        # print({"$set":data})
        ar=self.__tabobj.update(self.__where,{"$set":data},multi=multi)
        self.__None()
        return ar
    def delete(self,id=None):
        """文档删除 删除条件是where函数
        """
        self.__setconn()
        if id:
            self.where('_id',id)
        if self.__where:
            # print(self.__where)
            # exit()
            bo=self.__tabobj.remove(self.__where)
            self.__None()
            if bo:
                return bo['n']
            else:
                return 0
        else:
            self.__None()
            return 0
    def deleteAll(self,id=None):
        """删所有文档除
        """
        self.__setconn()
        bo=self.__tabobj.remove({})
        self.__None()
        if bo:
            return bo['n']
        else:
            return 0
    def insert(self,dicts):
        """插入文档 单条插入或多条插入

        参数 dicts 要插入的内容 单条格式：{"name":"测试","age":20}  。     多条格式：[{"name":"测试","age":20},{"name":"测试","age":20}]

        返回插入的数量
        
        """
        self.__setconn()
        co=0
        if isinstance(dicts,dict):
            if self.__tabobj.insert_one(dicts):
                co=1
        elif isinstance(dicts,list):
            lens=len(dicts)
            if lens>100:
                raise RuntimeError('列表数量超过最大限制100')
            if self.__tabobj.insert_many(dicts):
                co=lens
        return co
    __table=""
    def table(self,table):
        """设置集合名

        参数 table：str 表名
        """
        self.__table=table
        return self
    def __None(self):
        "清除所有赋值条件"
        # self.__lock=None
        # self.__distinct=None
        # self.__join=None
        # self.__joinstr=''
        # self.__alias=None
        # self.__having=None
        # self.__group=None
        # self.__group1=None
        # self.__order=None
        # self.__order1=None
        mongo.__limit=[]
        mongo.__field={}
        mongo.__where={}
        mongo.__table=None
        mongo.__order=None
    __where={}
    def where(self,where = None,*wheres):
        """设置过滤条件  

        参数 where：str 字符串 或 列表
       
        传入方式:

        "id",2 表示id='2'

        "id","in",2,3,4 ...表示 id=2 or id=3 or id=4 ...

        "id","or",2,3,4 ...表示 id=2 or id=3 or id=4 ...

        "id","neq",1 表示 id 不等于 '1'

        eq 等于
        neq 不等于
        gt 大于
        egt 大于等于
        lt 小于
        elt 小于等于
        like LIKE
        """
        # print("wheres",wheres)  {'comments':re.compile('abc')}
        if isinstance(where,dict):
            self.__where=where
        elif isinstance(where,list):
            # import re [("name","eq",'冯坤'),"and",("aa","like",'%wfweaf')]
            # print(re.compile('abc'))
            #{"likes": {$gt:50}, "name": "冯坤","title": "MongoDB 教程"}
            #{"likes":'dav', $or: [{"by": "菜鸟教程"},{"title": "MongoDB 教程"}]}
            zd={}
            t=''
            for k in where:
                if isinstance(k,tuple):
                    if k[1]=='eq':
                        if t=='or':
                            zd['$or'].append({k[0]:k[2]})
                        else:
                            zd[k[0]]=k[2]
                    elif k[1]=='like':
                        if t=='or':
                            zd['$or'].append({k[0]:re.compile(re.sub('%','',k[2]))})
                        else:
                            zd[k[0]]=re.compile(re.sub('%','',k[2]))
                    else:
                        if t=='or':
                            zd['$or'].append({k[0]:{'$'+k[1]:k[2]}})
                        else:
                            n=self.__operator(k[1])
                            zd[k[0]]={n:k[2]}
                elif isinstance(k,str):
                    if k=='or':
                        t=k
                        zd['$or']=[]
            self.__where=zd
            # print(zd)
            # exit()
        elif isinstance(where,str) and len(wheres)==1:
            wheres=list(wheres)
            if where=='_id':
                wheres[0]=ObjectId(wheres[0])
            self.__where[where]=wheres[0]
        elif isinstance(where,str) and len(wheres)==2:
            wheres=list(wheres)
            if where=='_id':
                wheres[1]=ObjectId(wheres[1])
            if wheres[0] == 'eq':
                self.__where[where]=wheres[1]
            elif wheres[0]=='like':
                 self.__where[where]=re.compile(re.sub('%','',wheres[1]))
            else:
                n=self.__operator(wheres[0])
                self.__where[where]={n:wheres[1]}
        elif isinstance(where,str) and isinstance(wheres,tuple):
            #{$or: [{key1: value1}, {key2:value2}]}
            # self.__where={'$or': [{where: wheres[0]}, {where:wheres[1]}]}
            # print(wheres)
            lists=[]
            for k in wheres:
                lists.append({where:k})
            self.__where={'$or': lists}
        # print(self.__where)
        return self
    __field={}
    def field(self,field = "*"):
        """设置过滤显示条件

        参数 field：str 字符串
        """
        if field and field!='*':
            field=field.split(",")
            zd={}
            for f in field:
                zd[f]=1
            self.__field=zd
        return self
    __limit=[]
    def limit(self,offset, length = None):
        """设置查询数量

        参数 offset：int 起始位置

        参数 length：int 查询数量
        """
        offset=int(offset)
        length=int(length)
        if length==None:
            length=offset
            offset=0
        # elif offset > 0:
        #     offset=offset*length-length
        self.__limit=[offset,length]
        return self
    def page(self,offset, length = None):
        """设置分页查询

        参数 offset：int 页码

        参数 length：int 页面数量
        """
        offset=int(offset)
        length=int(length)
        if length==None:
            length=offset
            offset=0
        elif offset > 0:
            offset=offset*length-length
        self.__limit=[offset,length]
        return self
    def __operator(self,strs):
        """运算符转换
        参数 strs 待转的字符串
        返回 已转换的运算符

        符号定义
            eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
        """
        strss=strs.upper()
        if strss == 'NEQ':
            k='$ne'
        elif strss == 'GT':
            k='$gt'
        elif strss == 'EGT':
            k='$gte'
        elif strss == 'LT':
            k='$lt'
        elif strss == 'ELT':
            k='$lte'
        else:
            k=strss
        return k


# -*- coding: utf-8 -*-
import os,sys,time,hashlib,json
from kcweb import config
from kcweb.utill import rediss as red
# from kcweb.utill.db.mysql import mysql

import time,hashlib
def md5(strs):
    """md5加密"""
    if not strs:
        return strs
    m = hashlib.md5()
    b = strs.encode(encoding='utf-8')
    m.update(b)
    return m.hexdigest()
def times():
    """时间戳 精确到秒"""
    return int(time.time())
def json_decode(jsonstr):
    """json字符串转python类型"""
    try:
        return eval(jsonstr)
    except Exception:
        return {}
# def json_decode(strs):
#     """json字符串转python类型"""
#     try:
#         return json.loads(strs)
#     except Exception:
#         return {}
class cache:
    "开发完善中..."
    __name=None
    __values=None
    __cachepath='' #os.path.split(os.path.realpath(__file__))[0]+'/../../../'
    __config=config.cache
    __redisobj=None
    __mysqlobj=None
    # def __setmysqlonj(self):
    #     conf=config.database
    #     if 'host' in self.__config and self.__config['host']:
    #         conf['host']=[self.__config['host']]
    #     if 'port' in self.__config and self.__config['port']:
    #         conf['port']=[self.__config['port']]
    #     if 'user' in self.__config and self.__config['user']:
    #         conf['user']=[self.__config['user']]
    #     if 'password' in self.__config and self.__config['password']:
    #         conf['password']=[self.__config['password']]
    #     if 'db' in self.__config and self.__config['db']:
    #         conf['db']=[self.__config['db']]
    #     db=mysql()
    #     self.__mysqlobj=db.connect(conf)
    def __setredisobj(self):
        "设置redis链接实例"
        conf=config.redis
        if 'host' in self.__config and self.__config['host']:
            conf['host']=self.__config['host']
        if 'port' in self.__config and self.__config['port']:
            conf['port']=self.__config['port']
        if 'password' in self.__config and self.__config['password']:
            conf['password']=self.__config['password']
        if 'db' in self.__config and self.__config['db']:
            conf['db']=self.__config['db']
        if conf['pattern']:
            if conf['password']:
                redis_pool=red.ConnectionPool(host=conf['host'],password=conf['password'],port=conf['port'],db=conf['db'])
            else:
                redis_pool=red.ConnectionPool(host=conf['host'],port=conf['port'],db=conf['db'])
            self.__redisobj=red.Redis(connection_pool=redis_pool)
        else:
            if conf['password']:
                self.__redisobj=red.Redis(host=conf['host'],password=conf['password'],port=conf['port'],db=conf['db'])
            else:
                self.__redisobj=red.Redis(host=conf['host'],port=conf['port'],db=conf['db'])
    def set_cache(self,name,values,expire = 'no'):
        """设置缓存

        参数 name：缓存名

        参数 values：缓存值

        参数 expire：缓存有效期 0表示永久  单位 秒
        
        return Boolean类型
        """
        # print(name)
        # exit()
        self.__name=name
        self.__values=values
        if expire != 'no':
            self.__config['expire']=int(expire)
        return self.__seltype('set')
    def get_cache(self,name):
        """获取缓存

        return 或者的值
        """
        self.__name=name
        return self.__seltype('get')
    def del_cache(self,name):
        """删除缓存

        return Boolean类型
        """
        self.__name=name
        return self.__seltype('del')
    def set_config(self,congig):
        """设置缓存配置
        """
        self.__config=congig
        return self

    
    def __seltype(self,types):
        """选择缓存"""
        # m = hashlib.md5()
        # b = self.__name.encode(encoding='utf-8')
        # m.update(b)
        self.__name=md5(self.__name)
        if self.__config['type'] == 'File':
            if types == 'set':
                return self.__setfilecache()
            elif types=='get':
                return self.__getfilecache()
            elif types=='del':
                return self.__delfilecache()
        elif self.__config['type'] == 'Redis':
            self.__setredisobj()
            if types == 'set':
                return self.__setrediscache()
            elif types=='get':
                return self.__getrediscache()
            elif types=='del':
                return self.__delrediscache()
        # elif self.__config['type'] == 'MySql':
        #     self.__setmysqlonj()
        #     if types == 'set':
        #         return self.__setmysqlcache()
        #     elif types == 'get':
        #         return self.__getmysqlcache()
        #     elif types == 'del':
        #         return self.__delmysqlcache()
        elif self.__config['type'] == 'Python':
            if types == 'set':
                return self.__setpythoncache()
            elif types == 'get':
                return self.__getpythoncache()
            elif types == 'del':
                return self.__delpythoncache()
        else:
            raise Exception("缓存类型错误")
    def __setpythoncache(self):
        """设置python缓存
        
        return Boolean类型
        """
        data={
            'expire':self.__config['expire'],
            'time':times(),
            'values':self.__values
        }
        config.cachevalue[self.__name]=data
        return True
    def __getpythoncache(self):
        """获取python缓存
        
        return 缓存的值
        """
        try:
            ar=config.cachevalue[self.__name]
        except KeyError:
            return ""
        else:
            if ar['expire'] > 0:
                if (times()-ar['time']) > ar['expire']:
                    self.__delpythoncache()
                    return ""
                else:
                    return ar['values']
            else:
                return ar['values']
    def __delpythoncache(self):
        """删除python缓存
        
        return Boolean类型
        """
        try:
            del config.cachevalue[self.__name]
        except KeyError:
            pass
        return True
    # def __setmysqlcache(self): ########################################################################################
    #     """设置mysql缓存
        
    #     return Boolean类型
    #     """
    #     data=[str(self.__values)]
    #     strs="["
    #     for k in data:
    #         strs=strs+k
    #     strs=strs+"]"
    #     k=self.__mysqlobj.table('fanshukeji_core_cache').where("name",self.__name).count('id')
    #     self.__setmysqlonj()
    #     if k:
    #         return self.__mysqlobj.table('fanshukeji_core_cache').where("name",self.__name).update({"val":strs,"expire":self.__config['expire'],"time":times()})
    #     else:
    #         return self.__mysqlobj.table('fanshukeji_core_cache').insert({"name":self.__name,"val":strs,"expire":self.__config['expire'],"time":times()})
    # def __getmysqlcache(self):
    #     """获取mysql缓存
        
    #     return 缓存的值
    #     """
    #     data=self.__mysqlobj.table('fanshukeji_core_cache').where("name",self.__name).find()
    #     if data :
    #         if data['expire']>0 and times()-data['time']>data['expire']:
    #             self.__setmysqlonj()
    #             self.__mysqlobj.table('fanshukeji_core_cache').where("name",self.__name).delete()
    #             return False
    #         else:
    #             return eval(data['val'])[0]
    #     else:
    #         return False
    # def __delmysqlcache(self):
    #     """删除mysql缓存
        
    #     return Boolean类型
    #     """
    #     return self.__mysqlobj.table('fanshukeji_core_cache').where("name",self.__name).delete()
    def __setrediscache(self):
        """设置redis缓存
        
        return Boolean类型
        """
        # print(self.__redisobj)
        data=[self.__values]
        try:
            if self.__config['expire']:
                self.__redisobj.set(self.__name,str(data),self.__config['expire'])
            else:
                self.__redisobj.set(self.__name,str(data))
        except:
            return False
        return True
    def __getrediscache(self):
        """获取redis缓存
        
        return 缓存的值
        """
        lists=self.__redisobj.get(self.__name)
        if lists:
            data=eval(lists)
            return data[0]
        else:
            return False
    def __delrediscache(self):
        """删除redis缓存
        
        return int类型
        """
        return self.__redisobj.delete(self.__name)
    def __setfilecache(self):
        """设置文件缓存
        
        return Boolean类型
        """
        data={
            'expire':self.__config['expire'],
            'time':times(),
            'values':self.__values
        }
        if not os.path.exists(self.__config['path']):
            os.makedirs(self.__config['path']) #多层创建目录
        f=open(self.__config['path']+"/"+self.__name,"w")
        f.write(str(data))
        f.close()
        return True
    def __getfilecache(self):
        """获取文件缓存
        
        return 缓存的值
        """
        try:
            f=open(self.__config['path']+"/"+self.__name,"r")
        except Exception:
            return ""
        json_str=f.read()
        f.close()
        ar=json_decode(json_str)
        
        if ar['expire'] > 0:
            if (times()-ar['time']) > ar['expire']:
                
                self.__delfilecache()
                return ""
            else:
                return ar['values']
        else:
            return ar['values']
    def __delfilecache(self):
        """删除文件缓存
        
        return Boolean类型
        """
        if not os.path.exists(self.__config['path']+"/"+self.__name):
            return True
        try:
            os.remove(self.__config['path']+"/"+self.__name)
        except:
            return False
        return True


# -*- coding: utf-8 -*-
from kcweb.config import sqlite as sqliteconfig
import time,traceback,re
import random,sqlite3,os,hashlib
class sqlite:
    __config=sqliteconfig
    __configt={}
    __conn={}
    __cursor={}
    __sql=None
    __sqls=None
    def md5(self,strs):
        """md5加密
        
        参数 strs：要加密的字符串

        return String类型
        """
        m = hashlib.md5()
        b = strs.encode(encoding='utf-8')
        m.update(b)
        return m.hexdigest()
    def close(self):
        "关闭连接"
        for k in sqlite.__conn.keys():
            sqlite.__cursor[k].close()
            sqlite.__conn[k].close()
        sqlite.__conn={}
        sqlite.__cursor={}
        sqlite.__configt={}
    def __setconn(self):
        if not self.__configt:
            self.__configt=sqliteconfig
        try:
            self.__conn[self.md5(self.__configt['db'])]
        except KeyError:
            try:
                if '/' in self.__configt['db']:
                    self.__conn[self.md5(self.__configt['db'])] = sqlite3.connect(self.__configt['db'],check_same_thread=False)
                else:
                    self.__conn[self.md5(self.__configt['db'])] = sqlite3.connect(os.path.split(os.path.realpath(__file__))[0]+"/sqlitedata/"+self.__configt['db'],check_same_thread=False)
            except Exception as e:
                raise Exception(e)
            self.__cursor[self.md5(self.__configt['db'])]=self.__conn[self.md5(self.__configt['db'])].cursor()
    def __execute(self,typess='DQL'):
        self.__setconn()
        # print(self.__sql)
        try:
            res=self.__cursor[self.md5(self.__configt['db'])].execute(self.__sql)
        except Exception as e:
            
            raise Exception(e)
        else:
            return res
    def connect(self,config):
        if isinstance(config,str):
            self.__configt['db']=config
        elif isinstance(config,dict):
            if 'db' in config:
                self.__configt['db']=config['db']
        return self
    __table=""
    def table(self,table):
        """设置表名

        参数 table：str 表名
        """
        self.__table=table
        return self
    def query(self,sql):
        self.__sql=sql
        self.__execute(sql,'DQL')
        # self.close()
    def execute(self,sql):
        self.__sql=sql
        res=self.__execute('DML')
        rowcount=res.rowcount
        # self.close()
        return rowcount
    # def create_table(self):
    #     self.__sql=("CREATE TABLE "+self.__table+
    #     "(ID INT PRIMARY KEY NOT NULL,"+
    #     "NAME TEXT NOT NULL,"+
    #     "AGE INT NOT NULL,"+
    #     "ADDRESS CHAR(50),"+
    #     "SALARY REAL);")
    #     # print(self.__sql)
    #     # exit()
    #     self.execute(self.__sql)
    def select(self,id=None):
        """select查询 

        返回 list(列表)
        """
        if id :
            self.__where="id=%d" % id
        self.__setsql()
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sqls
        self.__execute()
        description=self.__cursor[self.md5(self.__configt['db'])].description #获取字段
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        lists=[]
        data_dict=[]
        for field in description:#获取字段
            data_dict.append(field[0])
        for k in result:
            i=0
            dicts={}
            for j in k:
                dicts[data_dict[i]]=j
                i=i+1
            lists.append(dicts)
        self.__None(table=False)
        return lists
    def find(self,id=None):
        """查询一条记录
        
        返回 字典
        """
        if id :
            self.__where="id=%s" % id
        self.limit(1)
        self.__setsql()
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            self.__None()
            return self.__sqls
      
        self.__execute()
        description=self.__cursor[self.md5(self.__configt['db'])].description #获取字段
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        data_dict=[]
        for field in description:#获取字段
            data_dict.append(field[0])
        dicts={}
        for k in result:
            i=0
            for j in k:
                dicts[data_dict[i]]=j
                i=i+1
        self.__None(table=False)
        return dicts
    def count(self,field="*"):
        """查询数量
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('count')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            return self.__sql
        self.__execute()
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        if self.__group:
            cou=len(result)
        else:
            try:
                cou=int(result[0][0])
            except IndexError:
                cou=0
        self.__None(table=False)
        return cou
    def max(self,field):
        """查询某字段的最大值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('max')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            return self.__sql
        self.__execute()
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        cou=int(result[0][0])
        self.__None(table=False)
        return cou
    def min(self,field):
        """查询某字段的最小值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('min')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            return self.__sql
        self.__execute()
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        cou=int(result[0][0])
        self.__None(table=False)
        return cou
    def avg(self,field):
        """查询某字段的平均值
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('avg')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            return self.__sql
        self.__execute()
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        cou=int(result[0][0])
        self.__None(table=False)
        return cou
    def sum(self,field):
        """查询某字段之和
        
        返回 int 数字
        """
        self.__field=field
        self.__setsql('sum')
        if self.__buildSql:
            self.__sqls="("+self.__sql+")"
            return self.__sql
        self.__execute()
        result = self.__cursor[self.md5(self.__configt['db'])].fetchall() #获取查询结果
        # self.close()
        cou=int(result[0][0])
        self.__None(table=False)
        return cou
    def update(self,data,affair=False):
        """数据表更新
         
        参数 data 要更新的内容  格式：{"name":"测试","age":20}

        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
        """
        self.__setsql('update',data)
        res=self.__execute('DML')
        if affair==False and self.__startTrans==False:
            self.commit()
        rowcount=res.rowcount
        # self.close()
        self.__None(table=False)
        return rowcount
    def delete(self,affair=False):
        """数据表删除

        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务
        """
        self.__setsql('delete')
        if self.__where:
            res=self.__execute('DML')
        else:
            return 0
        if affair==False and self.__startTrans==False:
            self.commit()
        rowcount=res.rowcount
        # self.close()
        self.__None(table=False)
        return rowcount
    def insert(self,dicts,affair=False):
        """插入数据库 单条插入或多条插入

        参数 dicts 要插入的内容 单条格式：{"name":"测试","age":20}  。     多条格式：[{"name":"测试","age":20},{"name":"测试","age":20}]
        
        参数 affair 是否开启事务 True表示手动提交事务  False表示自动提交事务

        返回插入的数量
        """
        self.__setsql('insert',dicts)
        res=self.__execute('DML')
        if affair==False and self.__startTrans==False:
            self.commit()
        rowcount=res.rowcount
        # self.close()
        self.__None(table=False)
        return rowcount
    __startTrans=False
    def startTrans(self):
        "开启事务,仅对 update方法、delete方法、install方法有效"
        self.__startTrans=True
    def commit(self):
        """事务提交

        增删改后的任务进行提交
        """
        self.__conn[self.md5(self.__configt['db'])].commit()
    def rollback(self):
        """事务回滚

        增删改后的任务进行撤销
        """
        self.__conn[self.md5(self.__configt['db'])].rollback()
    def getsql(self):
        """得到生成的sql语句"""
        return self.__sql
    __buildSql=None
    def buildSql(self):
        """构造子查询"""
        self.__buildSql=True
        return self
    def __None(self,table=True):
        "清除所有赋值条件"
        self.__lock=None
        self.__distinct=None
        self.__join=None
        self.__joinstr=''
        self.__alias=None
        self.__having=None
        self.__group=None
        self.__group1=None
        self.__order=None
        self.__order1=None
        self.__limit=None
        self.__field="*"
        self.__where=None
        self.__wheres=()
        self.__buildSql=None
        if table:
            self.__table=None
    __where=None
    __wheres=()
    def where(self,where = None,*wheres):
        """设置过滤条件

        传入方式:
        "id",2 表示id='2'

        "id","in",2,3,4,5,6,...表示 id in (2,3,4,5,6,...)

        "id","in",[2,3,4,5,6,...]表示 id in (2,3,4,5,6,...)

        [("id","gt",6000),"and",("name","like","%超")] 表示 ( id > "6000" and name LIKE "%超" )

        "id","eq",1 表示 id = '1'

        eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
            like LIKE
        """
        self.__where=where
        self.__wheres=wheres
        # print(len(self.__wheres))
        return self
    __field='*'
    def field(self,field = "*"):
        """设置过滤显示条件

        参数 field：str 字符串
        """
        self.__field=field
        return self
    __limit=[]
    def limit(self,offset, length = None):
        """设置查询数量

        参数 offset：int 起始位置

        参数 length：int 查询数量
        """
        self.__limit=[offset,length]
        return self
    def page(self,pagenow=1, length = 20):
        """设置分页查询

        参数 pagenow：int 页码

        参数 length：int 查询数量
        """
        offset=(pagenow-1)*length
        self.__limit=[offset,length]
        return self
    __order=None
    __order1=None
    def order(self,strs=None,*strs1):
        """设置排序查询

        传入方式:

        "id desc"

        "id",'name','appkey','asc'

        "id",'name','appkey'   不包含asc或desc的情况下 默认是desc

        ['id','taskid',{"task_id":"desc"}]
        """
        self.__order=strs
        self.__order1=strs1
        return self
    __group=None
    __group1=None
    def group(self,strs=None,*strs1):
        """设置分组查询

        传入方式:

        "id,name"

        "id","name"
        """
        self.__group=strs
        self.__group1=strs1
        return self
    __having=None
    def having(self,strs=None):
        """用于配合group方法完成从分组的结果中筛选（通常是聚合条件）数据

        参数 strs：string 如："count(time)>3"
        """
        self.__having=strs
        return self
    
    __alias=None
    def alias(self,strs=None):
        """用于设置当前数据表的别名，便于使用其他的连贯操作例如join方法等。

        参数 strs：string 默认当前表作为别名
        """
        if strs:
            self.__alias=strs
        else:
            self.__alias=self.__table
        return self
    __join=None
    __joinstr=''
    def join(self,strs,on=None,types='INNER'):
        """用于根据两个或多个表中的列之间的关系，从这些表中查询数据

        参数 strs  string 如："test t1"   test表设置别名t1

        参数 on  string 如："t1.id=t2.pid"   设置连接条件

        参数 types  支持INNER、LEFT、RIGHT、FULL  默认INNER

        """
        joinstr=''
        if strs and on:
            joinstr=joinstr+types+" JOIN "+strs+" ON "+on+" "
        if joinstr:
            self.__joinstr=self.__joinstr+joinstr
        return self
    __distinct=None
    def distinct(self,bools=None):
        "用于返回唯一不同的值,配合field方法使用生效,消除所有重复的记录，并只获取唯一一次记录。"
        self.__distinct=bools
        return self
    __lock=None
    def lock(self,strs=None):
        """用于数据库的锁机制，在查询或者执行操作的时候使用  （暂未实现）

        排他锁 (Exclusive lock)

        共享锁 (Shared lock)
        
        参数 strs  如：True表示自动在生成的SQL语句最后加上FOR UPDATE，

        
        """
        # self.__lock=strs
        return self
    def __setsql(self,types=None,data = {}):
        """生成sql语句"""
        if types==None:
            self.__sql="SELECT"
            if self.__distinct and self.__field:
                self.__sql=self.__sql+" DISTINCT"
            if self.__alias:
                self.__sql=self.__sql+" %s FROM %s %s" % (self.__field,self.__table,self.__alias)
            else:
                self.__sql=self.__sql+" %s FROM %s" % (self.__field,self.__table)
        elif types=='count':
            self.__sql="SELECT COUNT(%s) FROM %s" % (self.__field,self.__table)
        elif types=='max':
            self.__sql="SELECT MAX(%s) FROM %s" % (self.__field,self.__table)
        elif types=='min':
            self.__sql="SELECT MIN(%s) FROM %s" % (self.__field,self.__table)
        elif types=='avg':
            self.__sql="SELECT AVG(%s) FROM %s" % (self.__field,self.__table)
        elif types=='sum':
            self.__sql="SELECT SUM(%s) FROM %s" % (self.__field,self.__table)
        elif types=='update':
            strs=''
            for k in data:
                if isinstance(data[k],str):
                    strs=strs+" %s = '%s' ," % (k,self.escape_string(data[k]))
                else:
                    strs=strs+" %s = %s ," % (k,data[k])
            strs=strs[:-1]
            self.__sql="UPDATE %s SET %s" % (self.__table,strs)
            # print(self.__sql)
        elif types=='delete':
            self.__sql="DELETE FROM %s" % (self.__table)
        elif types=='insert':
            if isinstance(data,dict):
                strs=''
                val=''
                for k in data:
                    strs=strs+"%s," % k
                    if isinstance(data[k],str):
                        val=val+"'%s'," % self.escape_string(data[k])
                    else:
                        val=val+"%s," % data[k]
                strs=strs[:-1]
                val=val[:-1]
                self.__sql="INSERT INTO "+str(self.__table)+" ("+strs+") VALUES ("+val+")"
                # print(self.__sql)
            elif isinstance(data,list):
                strs=''
                val='('
                for k in data[0]:
                    strs=strs+" , "+k
                for k in data:
                    for j in k:
                        if isinstance(k[j],str):
                            val=val+"'"+str(k[j])+"',"
                        else:
                            val=val+str(k[j])+","
                    val=val[:-1]
                    val=val+"),("
                val=val[:-2]
                self.__sql="INSERT INTO "+str(self.__table)+" ("+strs[3:]+") VALUES "+val
        
        if self.__joinstr:
            # print(self.__sql)
            self.__sql=self.__sql+" "+self.__joinstr
        if self.__where:
            if isinstance(self.__where,str):
                if self.__wheres:
                    if len(self.__wheres) == 2:
                        if isinstance(self.__wheres[1],list):
                            self.__sql=self.__sql + " WHERE %s %s (" % (self.__where,self.__operator(self.__wheres[0]))
                            for k in self.__wheres[1]:
                                self.__sql=self.__sql+str(k)+","
                            self.__sql=self.__sql[:-1]+")"
                        else:
                            self.__sql=self.__sql + " WHERE  %s %s '%s'" % (self.__where,self.__operator(self.__wheres[0]),self.__wheres[1])
                    elif len(self.__wheres) > 2:
                        if self.__wheres[0]=='in':
                            strs=str(self.__wheres[1])
                            i=0
                            for k in self.__wheres:
                                if i > 1:
                                    strs=strs+","+str(k)
                                i=i+1
                            self.__sql=self.__sql + " WHERE  %s in (%s)" % (self.__where,strs)
                    else:
                        self.__sql=self.__sql + " WHERE  %s = '%s'" % (self.__where,self.__wheres[0])
                else:
                    self.__sql=self.__sql + " WHERE  %s" % self.__where
            elif isinstance(self.__where,list):
                self.__sql=self.__sql + " WHERE  %s" % self.__listTrans()
            else:
                print("参数where类型错误",type(self.__where),self.__where)
        if self.__order:
            s=''
            if isinstance(self.__order,list):
                for strs in self.__order:
                    if isinstance(strs,str):
                        s=s+strs+","
                    else:
                        pass
                        for key in strs:
                            s=s+key+" "+strs[key]
                        s=s+","
                s=s[:-1]
            if isinstance(self.__order,str):
                if self.__order1:
                    if len(self.__order1) > 1:
                        if self.__order1[len(self.__order1)-1] == 'desc' or self.__order1[len(self.__order1)-1] == 'asc':
                            i=0
                            while i<len(self.__order1)-1:
                                s=s+self.__order1[i]+","
                                i=i+1
                            s=s[:-1]+" "+self.__order1[len(self.__order1)-1]
                        else:
                            for key in self.__order1:
                                s=s+key+","
                            s=s[:-1]
                            s=s+" asc"
                        s=self.__order+","+s
                    else:
                        s=s[:-1]+self.__order1[0]
                        s=self.__order+" "+s
                else:
                    s=self.__order
                # print(s)
            self.__sql=self.__sql+" ORDER BY "+s
        if self.__group:
            s=self.__group
            if self.__group1:
                for key in self.__group1:
                    s=s+","+key
            self.__sql=self.__sql+" GROUP BY "+s
        if self.__having:
            self.__sql=self.__sql+" HAVING "+self.__having
        if self.__limit:
            if self.__limit[1]:
                self.__sql=self.__sql+" LIMIT %d,%d" % (self.__limit[0],self.__limit[1])
            else:
                self.__sql=self.__sql+" LIMIT %d" % self.__limit[0]
        if self.__lock:
            if isinstance(self.__lock,str):
                self.__sql=self.__sql+" "+self.__lock
            else:
                self.__sql=self.__sql+' Exclusive lock'
        # print(self.__sql)
    def __listTrans(self):
        """列表转换sql表达式
        返回 字符串
        """
        strs=''
        #[('id', 'eq', '1'), 'or', ('id', 'eq', '2')]
        for k in self.__where:
            if isinstance(k,tuple):
                t=0
                for j in k:
                    if t==0:
                        strs=strs+' '+str(j)+' '
                    elif t==1:
                        strs=strs+self.__operator(j)
                    if t==2:
                        strs=strs+' "'+str(j)+'" '
                    t=t+1
            elif isinstance(k,str):
                strs=strs+k
        return "("+strs+")"
    def __operator(self,strs):
        """运算符转换
        参数 strs 待转的字符串
        返回 已转换的运算符

        符号定义
            eq 等于
            neq 不等于
            gt 大于
            egt 大于等于
            lt 小于
            elt 小于等于
            like LIKE
        """
        strss=strs.upper()
        if strss == 'EQ':
            k='='
        elif strss == 'NEQ':
            k='<>'
        elif strss == 'GT':
            k='>'
        elif strss == 'EGT':
            k='>='
        elif strss == 'LT':
            k='<'
        elif strss == 'ELT':
            k='<='
        elif strss == 'LIKE':
            k='LIKE'
        else:
            k=strss
        return k
    def escape_string(self,value):
        """sqlite3 字符串转义

        Value 字符串
        """
        # value = value.replace('/', '//')
        value = value.replace("'", "''")
        # value = value.replace('[', '/[')
        # value = value.replace(']', '/]')
        # value = value.replace('%', '/%')
        # value = value.replace('&', '/&')
        # value = value.replace('_', '/_')
        # value = value.replace('(', '/(')
        # value = value.replace(')', '/)')
        return value

    
    


# -*- coding: utf-8 -*-
import re
def oauth(url,config):
    data=False
    requeststr=re.sub('/','',url)
    if config.oauth['status']:
        for k in config.oauth['need']:
            if k == '*':
                data=True
            kk=k.split("*")
            if(len(kk)>1):
                s=re.sub('/','',kk[0])
                if s==requeststr[0:len(s)]:
                    data=True
                    break
            else:
                routestr= re.sub('/','',k)
                if routestr==requeststr:
                    data=True
                    break
        for k in config.oauth['unwanted']:
            if k == '*':
                data=False
            kk=k.split("*")
            if(len(kk)>1):
                s=re.sub('/','',kk[0])
                if s==requeststr[0:len(s)]:
                    data=False
                    break
            else:
                routestr= re.sub('/','',k)
                if routestr==requeststr:
                    data=False
                    break
    return data